//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCECanvas.h"
#import "OCEDocument.h"
#import "OCEWindowController.h"
#import "OCEPanel.h"
#import "OCEPanelObject.h"
#import "OCETrailObject.h"
#import "OCEPathObject.h"
#import "OCENavFrameObject.h"
#import "OCEMath.h"
#import "OCENavTrailObject.h"
#import "OCEModeView.h"
#import "OCEActiveComponent.h"

@implementation OCECanvas

- (id)initWithFrame:(NSRect)frameRect
{
    //there is also init stuff in the setWindowController Method
    [super initWithFrame:frameRect];

    // set Canvas
    float x = - frameRect.size.width/2.0;
    float y = - frameRect.size.height/2.0;
    canvasRect = NSMakeRect(x,y,frameRect.size.width,frameRect.size.height);
    
    // Display Mode
    displayMode = 0;
    editMode = 0; 
    
    
	// mouse events
	trackingRect = 0;
	
    // Zoom
    scale = 1.0;
    
    // accept dragged OCEPanel
    [self registerForDraggedTypes:[NSArray arrayWithObjects: OCEDragType, nil]];

    // welche objecte sind selected
    selectedObjects  = [[NSMutableArray arrayWithCapacity:0] retain];
    selectRect = NSZeroRect;
    selecting = NO;
    
    // stuff    
    fakePasteBoard = [[NSMutableArray arrayWithCapacity:0] retain];
    spaceIsPressed = NO;
    
    // Trail Stuff
    rubberBand = nil;
    focusRingObject = nil;
	newPath = nil;
    
    [NSBezierPath setDefaultLineJoinStyle:NSRoundLineJoinStyle]; 
	[NSBezierPath setDefaultLineCapStyle:NSButtLineCapStyle];
    // (oben) knnte irgendwo stehen. legt fest wie bezierPaths zusammengesetzt werden

    
    // Notifikation Stuff
    nc = [NSNotificationCenter defaultCenter];
    [nc addObserver: self selector: @selector(updateView:) name: @"canvasObjectsUpdated" object: nil];
    [nc addObserver: self selector: @selector(updateView:) name: @"navigationObjectsUpdated" object: nil];
    [nc addObserver: self selector: @selector(didUndo:) name: @"NSUndoManagerDidRedoChangeNotification" object: nil];
    [nc addObserver: self selector: @selector(didUndo:) name: @"NSUndoManagerDidUndoChangeNotification" object: nil];

	return self;
}

// Methoden fr direkten Aufruf
- (void)setWindowController:(OCEWindowController *)aController {
    controller = aController;
    document = [controller document];
    
    //continue to init stuff
    [controller setCanvasColor: [document canvasColor]];
    
    // set info-tabview
    [controller selectInfoTabViewItemAtIndex:0];
    
    // grid attributes
    [controller setGridSize: [document gridSize]];
    [controller setShowGrid: [document showGrid]];
    [controller setSnapToGrid: [document snapToGrid]];
	
	[controller setNavigationTools: [document navigationTool]];
    [controller setActiveComponentHint: [document activeComponentHint]];
}

- (OCEWindowController *)windowController 
{
    return controller;
}

- (OCEDocument *)document 
{
    return document;
}

- (BOOL)acceptsFirstResponder 
{
    return YES;
}

- (BOOL)becomeFirstResponder 
{
    return YES;
}

//  wenn man auf den view klickt bewegt sich nicht das fenster mit
- (BOOL)mouseDownCanMoveWindow 
{
    return NO;
}

// erhht die performance
- (BOOL)isOpaque 
{
    return YES; 
}

- (void) updateView: (NSNotification *)notification
{
    [self setNeedsDisplay:YES];
}

- (void) didUndo: (NSNotification *)notification
{
	[self clearSelection];
	[controller updateMainTabView];
	[self setNeedsDisplay:YES];
}

- (BOOL)isFlipped 
{
    return NO;
}

- (void)dealloc 
{
    [nc removeObserver: self];
    
    [fakePasteBoard release];
    [selectedObjects release];
		
    [super dealloc];
}

// ----------------------------------------------------------------------
// Methoden fr das Main Menu
// ----------------------------------------------------------------------
- (BOOL)validateMenuItem:(NSMenuItem*)anItem 
{

	if ([NSStringFromSelector([anItem action]) isEqualToString:@"displayModePopUpButtonAction:"]) 
		return YES;
	else if ([NSStringFromSelector([anItem action]) isEqualToString:@"selectVisible:"]) 
		return YES;
	else if ([NSStringFromSelector([anItem action]) isEqualToString:@"selectAll:"])
		return YES;
	else if ([[anItem title] isEqualToString:@"Paste"]) {
		if ([fakePasteBoard count] > 0)
			return YES;
		else
			return NO;
	}
	else if (([selectedObjects count] > 0) && (displayMode != 2))
		return YES;
	else
		return NO;
}

// ----------------------------------------------------------------------
// Canvas Methoden
// ----------------------------------------------------------------------
- (void)drawRect:(NSRect)rect
{
	[self updateCanvasRect:rect];
	// Hintergrundfarbe
	[[document canvasColor] set]; 
	NSRectFill(rect);

    if (displayMode == 0){
		[self drawEditMode:rect];
	} else {
            [self drawNavigationMode:rect];
    }
}

- (void)drawEditMode:(NSRect)rect 
{
    // Grid zeichnen
    if ([document showGrid]) {
        [self drawGridInRect:rect];
    }
    
    if (dragging) {
        [[NSColor grayColor] set];
        [NSBezierPath setDefaultLineWidth: 5];
        [NSBezierPath strokeRect: rect];
    }
    
    // translate damit sich das canvasRect ber den zu zeichnenden Objecten befindet
    NSAffineTransform *transform = [NSAffineTransform transform];
    [transform translateXBy: -canvasRect.origin.x yBy: -canvasRect.origin.y];
    [transform concat];
		
    NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
    
    // panels und trails zeichnen
	[self drawObjectsInRect: canvasRect];
	
	if (focusRingObject)
	{
		[currentContext saveGraphicsState];
		[focusRingObject drawFocusRing];
		[currentContext restoreGraphicsState];
	}
	
    if (rubberBand)
    {
        [[NSColor redColor] set];
        [rubberBand stroke];
    }

    // zeichne select rect
    if (selecting)
    {
        [[NSColor colorWithCalibratedRed:1.0 green: 0.0 blue: 0.0 alpha:0.25] set];
        [NSBezierPath fillRect:selectRect];
        [[NSColor redColor] set];
        [NSBezierPath setDefaultLineWidth:0.5];
        [NSBezierPath strokeRect:selectRect];
    }  
}

- (void)drawNavigationMode:(NSRect)rect
{	
    // translate damit sich das canvasRect ber den zu zeichnenden Objecten befindet
    NSAffineTransform *transform = [NSAffineTransform transform];
    [transform translateXBy: -canvasRect.origin.x yBy: -canvasRect.origin.y];
    [transform concat];
    
    NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
    
	// panels und trails zeichnen
	[self drawObjectsInRect: canvasRect];

    
    // lege weissen schleier ber panels
    [[NSColor colorWithCalibratedWhite:1.0 alpha:0.75] set];
    [NSBezierPath fillRect:canvasRect];
	
	// Grid zeichnen
    if ([document showGrid]) 
	{
		[transform invert];
		[transform concat];
        [self drawGridInRect:rect];
		[transform invert];
		[transform concat];

    }
    	
    
    // zeichne navigation
	NSMutableArray *objects = [document navigationObjects];
    OCEObject *object;
    unsigned int i = 0;
    while (i < [objects count]) 
    {
        object = [objects objectAtIndex:i];
        if (NSIntersectsRect([object bounds],canvasRect))
        {
            [currentContext saveGraphicsState];
            //[NSBezierPath clipRect:rect];
            [object drawObjectIsSelected:[self objectIsSelected:object]];
            [currentContext restoreGraphicsState];
        }
		i++;
    }
    
	if (focusRingObject)
	{
		[currentContext saveGraphicsState];
		[focusRingObject drawFocusRing];
		[currentContext restoreGraphicsState];
	}
	
    if (rubberBand)
    {
        [[NSColor redColor] set];
        [rubberBand stroke];
    }
	
	
    // zeichne select rect
    if (selecting)
    {
        [[NSColor colorWithCalibratedRed:1.0 green: 0.0 blue: 0.0 alpha:0.25] set];
        [NSBezierPath fillRect:selectRect];
        [[NSColor redColor] set];
        [NSBezierPath setDefaultLineWidth:0.5];
        [NSBezierPath strokeRect:selectRect];
    }        
}   



- (void) drawObjectsInRect: (NSRect) rect
{
	NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];

	// panels und trails zeichnen
    NSMutableArray *objects = [document canvasObjects];
    OCEObject *object;
    unsigned int i = 0;
    while (i < [objects count]) 
    {
        object = [objects objectAtIndex:i];
		if (NSIntersectsRect([object bounds],rect))
        {
            [currentContext saveGraphicsState];
            [object drawObjectWithFraction: 1 isSelected: [self objectIsSelected:object]];
            [currentContext restoreGraphicsState];
        }
		i++;
    }
}


- (void)updateCanvasRect:(NSRect) rect
{
	float width = rect.size.width / 2.0;
	float height = rect.size.height / 2.0;

	float x = NSMidX(canvasRect) - width;
	float y = NSMidY(canvasRect) - height;
	canvasRect = NSMakeRect(x,y,rect.size.width,rect.size.height);
}


- (NSRect) canvasRect
{
    return canvasRect;
}

- (NSPoint) convertPointToView: (NSPoint) aPoint
{
       
    float x,y;
    if (canvasRect.origin.x >= 0)
    {
        if (aPoint.x >= 0)
            x = aPoint.x - canvasRect.origin.x; // beide positiv
        else
            x = -(ABS(aPoint.x)+canvasRect.origin.x); // canvasRect positiv und aPoint negativ
    }
    else
    {
        x = ABS(canvasRect.origin.x) + aPoint.x; // beide negativ oder convasRect negativ und aPoint positiv
    }
    
    
    if (canvasRect.origin.y >= 0)
    {
        if (aPoint.y >= 0)
            y = aPoint.y - canvasRect.origin.y; // beide positiv
        else
            y = -(ABS(aPoint.y)+canvasRect.origin.y); // canvasRect positiv und aPoint negativ
    }
    else
    {
        y = ABS(canvasRect.origin.y) + aPoint.y; // beide negativ oder convasRect negativ und aPoint positiv
    }
    
    return NSMakePoint(x,y);
}

- (NSRect) convertRectToView: (NSRect) aRect
{
    NSPoint newOrigin = [self convertPointToView: aRect.origin];
    
    return NSMakeRect(newOrigin.x,newOrigin.y,aRect.size.width,aRect.size.height);
}

- (NSPoint) convertPointToModel: (NSPoint) aPoint
{	
	return NSMakePoint(canvasRect.origin.x+aPoint.x,canvasRect.origin.y+aPoint.y);
}

- (NSRect) convertRectToModel: (NSRect) aRect
{
    NSPoint aPoint = [self convertPointToModel:aRect.origin];
    return NSMakeRect(aPoint.x, aPoint.y,aRect.size.width,aRect.size.height);
}


- (bool) draggingInProgress
{
    return dragging;
}

- (OCEObject *) objectUnderPoint:(NSPoint)point 
{
    NSArray *objects = ((displayMode) ? [document navigationObjects] : [document canvasObjects]);
    int i = [objects count]-1;
    OCEObject *object = nil;
    
    for (; i> -1; i--) 
    {
        object = [objects objectAtIndex:i];
        if ([object hitTest:point isSelected: [self objectIsSelected:object]]) 
            break;
    }
    if (i > -1) 
        return object;
    else 
        return nil;
}

- (OCERectObject *) checkForFocusRingObject: (NSPoint)point isStartPoint:(bool) startPoint
{
    NSArray *objects = ((displayMode) ? [document navigationObjects] : [document canvasObjects]);
    int i = [objects count]-1;
    OCEObject *object = nil;
    
    for (; i> -1; i--) 
    {
        object = [objects objectAtIndex:i];
		if ([object isKindOfClass:[OCERectObject class]])
		{
			OCERectObject *returnObject;
			if (returnObject = [(OCERectObject *)object checkIfFocusRingObject:point isStartPoint: startPoint]) 
				return returnObject;
		}
    }
    return nil;
}

// ----------------------------------------------------------------------
// Select Methoden
// ----------------------------------------------------------------------
- (NSMutableArray *) selectedObjects
{
    return selectedObjects;
}

- (BOOL)objectIsSelected:(OCEObject *)object 
{
    return (([selectedObjects indexOfObjectIdenticalTo:object] == NSNotFound) ? NO : YES);
}

- (void)selectObject:(OCEObject *)object {
    unsigned curIndex = [selectedObjects indexOfObjectIdenticalTo:object];
    if (curIndex == NSNotFound) 
    {
        [selectedObjects addObject:object];
    }
}

- (void)deselectObject:(OCEObject *)object 
{
    unsigned curIndex = [selectedObjects indexOfObjectIdenticalTo:object];
    if (curIndex != NSNotFound) 
    {
        [selectedObjects removeObjectAtIndex:curIndex];
    }
}

- (void)clearSelection 
{
    [selectedObjects removeAllObjects];
}

- (bool) objectsAreSelected
{
	return ([selectedObjects count] ? YES : NO);
}

// ----------------------------------------------------------------------
// Grid Methoden
// ----------------------------------------------------------------------
- (void) drawGridInRect: (NSRect) rect
{
    int curLine, endLine;
    NSBezierPath *gridPath = [NSBezierPath bezierPath];
    [[document gridColor] set];
	float gridSize = [document gridSize];
    
    float x = floor(canvasRect.origin.x / gridSize); // starting position x
    x = x * gridSize;
    
    float y = floor(canvasRect.origin.y / gridSize); // starting position y
    y = y * gridSize;
    
    NSPoint gridOrigin = [self convertPointToView: NSMakePoint(x,y)];
    
    // Columns
    curLine = ceil((NSMinX(rect) - gridOrigin.x) / gridSize);
    endLine = floor((NSMaxX(rect) - gridOrigin.x) / gridSize);
    for (; curLine<=endLine; curLine++) {
        [gridPath moveToPoint:NSMakePoint((curLine * gridSize) + gridOrigin.x, NSMinY(rect))];
        [gridPath lineToPoint:NSMakePoint((curLine * gridSize) + gridOrigin.x, NSMaxY(rect))];
    }

    // Rows
    curLine = ceil((NSMinY(rect) - gridOrigin.y) / gridSize);
    endLine = floor((NSMaxY(rect) - gridOrigin.y) / gridSize);
    for (; curLine<=endLine; curLine++) 
    {
        [gridPath moveToPoint:NSMakePoint(NSMinX(rect), (curLine * gridSize) + gridOrigin.y)];
        [gridPath lineToPoint:NSMakePoint(NSMaxX(rect), (curLine * gridSize) + gridOrigin.y)];
    }

    [gridPath setLineWidth:0.75];
    [gridPath stroke];
    
    // draw origin
    NSBezierPath *originPath = [NSBezierPath bezierPath];
    NSPoint originPoint = [self convertPointToView: NSZeroPoint];
    [originPath moveToPoint:NSMakePoint(originPoint.x,NSMinY(rect))];
    [originPath lineToPoint:NSMakePoint(originPoint.x,NSMaxY(rect))];
     
    [originPath moveToPoint:NSMakePoint(NSMinX(rect), originPoint.y)];
    [originPath lineToPoint:NSMakePoint(NSMaxX(rect), originPoint.y)];        
        
    [originPath setLineWidth:1.5];
    [originPath stroke];
}



- (NSPoint) gridPoint: (NSPoint) aPoint
{
	// no need to grid when flag not set!
	if (![document snapToGrid])
		return aPoint;
	return [OCEObject gridPoint: aPoint withGridSize: [document gridSize]];
}

// ----------------------------------------------------------------------
// Display/Edit Mode Methoden
// ----------------------------------------------------------------------
- (void) setDisplayMode: (int) flag 
{
	if (trackingRect)
	{
		[self removeTrackingRect:trackingRect];
		trackingRect = 0;
	}

    if (displayMode != flag)
    {
		// was muss bei dem wechsel eines display modes gemacht werden
        // first clean up
        if ((displayMode == 0) || (displayMode == 1))
        {
            if (editMode == 1) 
			// falls wir im trail mode sind. damit das zeichnen das aktuellen trails abgebrochen wird
            {
                if(rubberBand)
				{
                    [rubberBand release];
					rubberBand = nil;
				}
                focusRingObject = nil;
				if (newPath)
					if (![newPath valid])
					{
						if (displayMode == 0)
							[document removeCanvasObject: newPath];
						else if (displayMode == 1)
							[document removeNavigationObject: newPath];
					}
            } 
        } else if (displayMode == 2) {
			[controller enableZoomSlider];
		}
        [selectedObjects removeAllObjects];
        
        // then change mode
        displayMode = flag;
        
        // and finally prepare new mode
        if ((displayMode == 0) || (displayMode == 1))
		{
            [self scaleCanvas: scale];
			if (editMode == 3)
			{
				if (!trackingRect)
					trackingRect = [self addTrackingRect: [self bounds] owner: self userData: nil assumeInside: NO];
			}
			
		}
        else
        {
			[controller disableZoomSlider];
        }
		[modeView setDisplayMode:displayMode];
        [controller updateMainTabView];
		[[self window] makeFirstResponder:self]; // this view should become first responder
		
		[fakePasteBoard removeAllObjects];
		[modeView setState:0];
        [self setNeedsDisplay:YES];
    }
}

- (int) displayMode 
{
    return displayMode;
}

- (void) setEditMode: (int) flag 
{
    if (editMode != flag)
    {
        if (editMode == 1) // falls wir im trail mode sind. damit das zeichnen das aktuellen trails abgebrochen wird
        {
            if(rubberBand)
			{
                [rubberBand release];
				rubberBand = nil;
			}
            focusRingObject = nil;
			if (newPath)
				if (![newPath valid])
				{
					if (displayMode == 0)
						[document removeCanvasObject: newPath];
					else if (displayMode == 1)
						[document removeNavigationObject: newPath];
				}
        }
    
        if ((editMode != 1) && (flag != 0)) // nicht bei bergang trail zeichnen -> selectieren resetten
        {
            [selectedObjects removeAllObjects]; // reset weil es in den anderen modes nicht bentigt wird
        }

		if (editMode == 3 )
		{
			if (trackingRect)
			{
				[self removeTrackingRect:trackingRect];
				trackingRect = 0;
			}
		}
					
			
        editMode = flag;
			
		if (editMode == 3 )
		{
			if (!trackingRect)
				trackingRect = [self addTrackingRect: [self bounds] owner: self userData: nil assumeInside: NO];
		}
			
        [self setNeedsDisplay:YES];
		[[self window] makeFirstResponder:self]; // this view should become first responder
		[controller updateMainTabView];
    }
}

- (int) editMode 
{
    return editMode;
}

// ----------------------------------------------------------------------
// Scale Canvas
// ----------------------------------------------------------------------
- (void) scaleCanvas: (float) newFactor 
{
    NSAffineTransform *at = [NSAffineTransform transform];
    [at scaleBy:newFactor];
    NSSize newBoundsSize = [at transformSize:[self frame].size];
    [self setBoundsSize:newBoundsSize];
    [self setNeedsDisplay:YES];
    scale = newFactor;
}


- (float) scaleCanvasFactor
{
    return scale;
}


// ----------------------------------------------------------------------
// DRAG DESTINATION METHODEN
// ----------------------------------------------------------------------

// curser setzen
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender 
{
	NSPasteboard *pasteBoard = [sender draggingPasteboard];
	if ( [[pasteBoard types] containsObject:OCEDragType] ) 
	{
		dragging = YES;
		[self setNeedsDisplay:YES];
		return NSDragOperationLink;
	}
    return NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    dragging = NO;
    [self setNeedsDisplay:YES];
}

// ja, ich bin bereit :)
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    return YES;
}

// drag operation durchfhren 
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	if (!(displayMode == 0))
		[controller setMode:0];
	
	if (!(editMode == 0))
		[modeView setState:0];
	
    NSPasteboard *pasteBoard = [sender draggingPasteboard];
    
    if ( [[pasteBoard types] containsObject:OCEDragType] ) 
    {
        [selectedObjects removeAllObjects];
		
        // restore Pointer
        NSNumber *value = [NSUnarchiver unarchiveObjectWithData:[pasteBoard dataForType:OCEDragType]];
        NSMutableArray *panelsList = (NSMutableArray *) [value intValue];
        
        int i = 0;
		int offset = 10;
		if ([document snapToGrid])
			offset = [document gridSize];
		
        NSPoint imagePoint = [self convertPoint:[sender draggedImageLocation] fromView:nil];
        imagePoint = [self gridPoint: [self convertPointToModel:imagePoint]]; // ursprung des draggedImage
        
        OCEPanel *panel;
        NSEnumerator *panelsEnumerator = [panelsList objectEnumerator];
        while(panel = [panelsEnumerator nextObject]) 
        {
            // add to canvas
            if (i)
            {
                imagePoint.x += offset;
                imagePoint.y -= offset;
            }
            OCEPanelObject *newObject = [[OCEPanelObject alloc] initWithRect:(NSRect) {imagePoint,[[panel image] size]}];
            [newObject setPanel:panel];
            [newObject setUndoManager:[document undoManager]];
            // [selectedObjects addObject: newObject]; don't select new panel - off since 1.0.1
			if ([document snapToGrid])
				[newObject gridObject:[document gridSize]];
			
            [document addCanvasObject:newObject];
            [newObject release];
            i++;
        }
        [self setNeedsDisplay:YES];
		[[self window] makeFirstResponder:self]; // this view should become first responder
												 // [controller updateMainTabView]; don't switch to info panel - off since 1.0.1
        return YES;
    }
    else
    {
        return NO; // keine passenden filetypen gefunden
    }
	
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    dragging=NO;
    [self setNeedsDisplay:YES];
}

// ----------------------------------------------------------------------
// Mouse Tracking Methoden
// ----------------------------------------------------------------------
- (void)selectAndTrackMouseWithEvent:(NSEvent *)theEvent {
    NSPoint curPoint;
    OCEObject *object = nil;
    BOOL isSelected;
    BOOL modifier = (([theEvent modifierFlags] & NSCommandKeyMask) ? YES : NO);
    
    curPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
    object = [self objectUnderPoint:curPoint];
    isSelected = (object ? [self objectIsSelected:object] : NO);
    
    if (!modifier && !isSelected) 
    {
        [self clearSelection];
    }
	
	if (spaceIsPressed)
	{
		[self moveCanvasWithEvent:theEvent];
		return;
	}
    else if (object) 
    {
        // Add or remove this graphic from selection.
        if (modifier) 
        {
            if (isSelected) 
            {
                [self deselectObject:object];
                isSelected = NO;
            } 
            else 
            {
                [self selectObject:object];
                isSelected = YES;
            }
        } 
        else 
        {
            if (isSelected) 
            {
                int knobHit = [object hitTest:curPoint isSelected:isSelected];
                if ((knobHit != NoKnob) && (knobHit != Object)) 
                {
                    [self trackKnob:knobHit ofObject:object withEvent:theEvent];
                    return;
                }
            }
            else
            {
                [self selectObject:object];
                isSelected = YES;
            }
        }
    } 
    else 
    {
		[self selectObjectsWithEvent:theEvent];
        return;
    }
    
    if (isSelected) 
    {
        [self moveSelectedObjectsWithEvent:theEvent];
        return;
    }
    
    //  Just track until mouseUp:.
    [self setNeedsDisplay:YES];
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
        if ([theEvent type] == NSLeftMouseUp) 
        {
            break;
        }
    }
}

- (void)moveCanvasWithEvent:(NSEvent *)theEvent 
{
    NSPoint curPoint, lastPoint;
	float factor = 1;
	
    lastPoint = curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    BOOL checkSpace = NO;
    
    if (spaceIsPressed)
        checkSpace = YES;
	
	// dont use in pre10.3
	if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
	{
		[[NSCursor closedHandCursor] set];
	}
	
	
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
        curPoint =  [self convertPoint:[theEvent locationInWindow] fromView:nil];
        canvasRect.origin.x += (lastPoint.x - curPoint.x)*factor;
        canvasRect.origin.y += (lastPoint.y - curPoint.y)*factor;
        [self setNeedsDisplay:YES];
        [controller updateMainTabView];
        
        if ([theEvent type] == NSLeftMouseUp) 
            break;
        
        if ((checkSpace) && (!spaceIsPressed)) // exit wenn space nicht mehr gedrckt wird
            break;
        
        lastPoint = curPoint;
    }
	
	if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
	{
		if ((editMode == 3) || (spaceIsPressed) || (displayMode == 2))
			[[NSCursor openHandCursor] set];
	}
}

- (void)selectObjectsWithEvent:(NSEvent *)theEvent 
{
    NSPoint origPoint, curPoint;
    NSEnumerator *objectsEnumerator;
    NSArray *objects = ((displayMode) ? [document navigationObjects] : [document canvasObjects]);
    NSMutableArray *selectedObjectsSave = nil;
    OCEObject *object;
    BOOL modifier = (([theEvent modifierFlags] & NSCommandKeyMask) ? YES : NO);

    if (modifier)
        selectedObjectsSave = [selectedObjects mutableCopy]; // this is not deep copy!
    
    selecting = YES;
    origPoint = curPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
    
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
        curPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
        float width  = curPoint.x - origPoint.x;
        float height = curPoint.y - origPoint.y;
        selectRect.origin = origPoint;
        
        if (width < 0)
        {
            selectRect.origin.x += width;
            selectRect.size.width = ABS(width);
        }
        else
        {
            if (width == 0)
                selectRect.size.width = 1; // damit trotzdem etwas angezeigt wird!
            else
                selectRect.size.width = width;                       
        }
        
        if (height < 0)
        {
            selectRect.origin.y += height;
            selectRect.size.height = ABS(height);
        }
        else
        {
            if (height == 0)
                selectRect.size.height = 1; // damit trotzdem etwas angezeigt wird!
            else
                selectRect.size.height = height;                       
        }                        
        
        if (modifier)
        {
            [selectedObjects release];
            selectedObjects = [selectedObjectsSave mutableCopy];
        }
        else
        {
            [self clearSelection];   
        }
        
        objectsEnumerator = [objects objectEnumerator];
        while(object = [objectsEnumerator nextObject]) 
        {
            if ([object intersectsRect:selectRect])
            {
                if (modifier)
                    [self deselectObject:object];
                else
                    [self selectObject:object];
            }
        }
        [self setNeedsDisplay:YES];
        [controller updateMainTabView];

        if ([theEvent type] == NSLeftMouseUp) 
        {
            break;
        }
    }
    
    selecting = NO;
    selectRect = NSZeroRect;
    if (modifier)
        [selectedObjectsSave release];
}

- (void)moveSelectedObjectsWithEvent:(NSEvent *)theEvent 
{
    NSPoint lastPoint, curPoint, origin, offset, oldOrigin;
    unsigned i, c;
    OCEObject *object;
    c = [selectedObjects count];
    NSPoint realOrigins[c], origins[c];
    for (i=0; i<c; i++) 
    {
		object = [selectedObjects objectAtIndex:i];

		if (([document snapToGrid]) && ([object snapAble]))
			[object gridObject:[document gridSize]];
        realOrigins[i] = [object origin];
		origins[i] = realOrigins[i];
		[object startManipulationWithSelector:@selector(moveXBy:moveYBy:) andData: nil];
    }
    
    lastPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
        curPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
        offset = NSMakePoint(curPoint.x - lastPoint.x, curPoint.y - lastPoint.y);
        if (!NSEqualPoints(lastPoint, curPoint)) 
        {
            for (i=0; i<c; i++) 
            {
                object = [selectedObjects objectAtIndex:i];
                oldOrigin = origins[i];
				origin = realOrigins[i];
                origin.x += offset.x;
                origin.y += offset.y;
                realOrigins[i] = origin;
				
                if (([document snapToGrid]) && ([object snapAble]))
                    origin = [self gridPoint:origin];
				
                [object moveXBy: origin.x-oldOrigin.x moveYBy: origin.y-oldOrigin.y];
				oldOrigin.x += origin.x-oldOrigin.x;
                oldOrigin.y += origin.y-oldOrigin.y;
				origins[i] = oldOrigin;				
            }
            lastPoint = curPoint;
            [self setNeedsDisplay:YES];
            [controller updateMainTabView];
        }
        if ([theEvent type] == NSLeftMouseUp) 
        {
            [self setNeedsDisplay:YES];
            [controller updateMainTabView];
            break;
        }
    }
    
    // register undo
    [selectedObjects makeObjectsPerformSelector:@selector(stopManipulation)];
}

- (void)trackKnob:(int)knob ofObject:(OCEObject *)object withEvent:(NSEvent *)theEvent 
{
    NSPoint point, lastPoint;
    lastPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
	NSPoint realPoint = [object pointForKnob: knob];
	float mouseXOffset = realPoint.x - lastPoint.x;
	float mouseYOffset = realPoint.y - lastPoint.y;
	lastPoint.x += mouseXOffset;
	lastPoint.y += mouseYOffset;

	
	// Path?
	bool isPath = NO;
	bool isStartPoint = NO;
	if (([object isKindOfClass: [OCEPathObject class]]) && ((knob-2 == 2) || ([(OCEPathObject *) object endPointAtKnob:knob])))
	{
		isPath = YES;
		isStartPoint = (knob-2 == 2) ? YES : NO;
	}
    [object startManipulationWithSelector:@selector(movingKnob:fromPoint:toPoint:withEvent:) 
								  andData: [NSNumber numberWithInt: knob]];
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
        point = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];

		if (!NSEqualPoints(lastPoint,point)) // falls punkte gleich sind ist nichts zu tun
		{
			if (isPath)
				focusRingObject = [self checkForFocusRingObject: point isStartPoint: isStartPoint];
			
			point.x += mouseXOffset;
			point.y += mouseYOffset;
			if (([document snapToGrid])  && ([object snapAble]))
				if ((isPath) || !([theEvent modifierFlags] & NSControlKeyMask))
					point = [self gridPoint: point]; // don't grid if we need to rotate
			
			knob = [object movingKnob:knob fromPoint: lastPoint toPoint:point withEvent:theEvent];
			
			[controller updateMainTabView];
			[self setNeedsDisplay:YES];
		}
		lastPoint = point;

		// check abbruchbedingung
        if ([theEvent type] == NSLeftMouseUp) 
        {
			if (isPath)
			{
				if (focusRingObject)
				{
					if (isStartPoint)
						[(OCEPathObject *)object setAssociatedStartRect: focusRingObject];
					else
						[(OCEPathObject *)object setAssociatedEndRect: focusRingObject];
				}
				focusRingObject = nil;
			}
            break;
        }
    }
	
    [object stopManipulation];
}

- (void) createFrameWithEvent: (NSEvent *) theEvent
{
	NSPoint point, lastPoint;
    lastPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
    
	if ([document snapToGrid])
		lastPoint = [self gridPoint: lastPoint];
	
	OCENavFrameObject *newFrame = [[OCENavFrameObject alloc] initWithRect: (NSRect) {lastPoint,NSZeroSize}];
	[document addNavigationObject:newFrame];
	[newFrame release];
	[newFrame setUndoManager:[document undoManager]];
	
	
	[self clearSelection];
	[selectedObjects addObject: newFrame]; // select new Object
	[controller updateMainTabView];
	
	int knob = LowerRightKnob;
	
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
        if ([theEvent type] == NSLeftMouseUp) // set endpoint
            break;
		
        point = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
		if ([document snapToGrid])
			point = [self gridPoint: point];
		
		knob = [newFrame movingKnob:knob fromPoint: lastPoint toPoint:point withEvent: theEvent];
		lastPoint = point;
        [controller updateMainTabView];
        [self setNeedsDisplay:YES];
    }
	newPath = nil;
	[nc postNotificationName: @"frameCreated" object: nil];  
	[self setNeedsDisplay:YES];
}

- (void) createPathWithEvent: (NSEvent *) theEvent
{
    NSPoint point, lastPoint;
    lastPoint = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
    focusRingObject = [self checkForFocusRingObject: lastPoint isStartPoint:YES];
	
    if (displayMode == 0)
	{
		lastPoint = [self gridPoint: lastPoint];
		newPath = [[OCETrailObject alloc] initWithPoint: lastPoint];
		//[(OCETrailObject *) newPath setArrowImage:[document arrowImage]];
		[document addCanvasObject:newPath];
	}
	else
	{
		/*if (! focusRingObject)
			return;*/ // removed for IC1.2 (ein nav trail muss unter einem object anfangen)
		newPath = [[OCENavTrailObject alloc] initWithPoint: lastPoint];
		[document addNavigationObject:newPath];
		//[newPath setColor:[[(OCENavFrameObject *)focusRingObject borderColor] retain]];
	}
	
	[newPath setUndoManager:[document undoManager]];
	rubberBand = [[NSBezierPath bezierPath] retain];
	[rubberBand setLineWidth:1];
	
	if  (focusRingObject)
	{
		[newPath setAssociatedStartRect: focusRingObject];
		NSPoint center = [focusRingObject center];
		[rubberBand moveToPoint:center];
		[rubberBand lineToPoint:lastPoint];
	}
	else
	{
		[rubberBand moveToPoint:lastPoint];
		[rubberBand lineToPoint:lastPoint];
	}
	
	[self clearSelection];
	[selectedObjects addObject: newPath]; // select new Object
	[controller updateMainTabView];
	[newPath release];
	
	
    while (1) 
    {
        theEvent = [[self window] nextEventMatchingMask:(NSMouseMovedMask | NSLeftMouseDownMask | NSKeyDownMask)];
        if (([theEvent type] == NSLeftMouseDown) && ([theEvent clickCount]==2) /*&& (displayMode == 0) removes IC1.2*/) // set endpoint
            break; // der end punkt eines nav trails darf nur ber einen rect liegen. deshalb darf hier nicht
					// ausgestiegen werden

        point = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
		point = [self gridPoint:point];
		focusRingObject = [self checkForFocusRingObject: point isStartPoint:NO];
		
		if ([theEvent type] == NSKeyDown) 
			if ([theEvent keyCode] == 53)// escape
				break;
		
		if ([theEvent type] == NSLeftMouseDown) 
        {
			if (displayMode == 0)
				point = [self gridPoint: point];
			
            [newPath addLineToPoint: point];
			
			// check if associated endpoint
			if (focusRingObject)
			{
				[newPath setAssociatedEndRect: focusRingObject];
				break;
			}
			
            [rubberBand setAssociatedPoints: &point atIndex: 0];
            [rubberBand setAssociatedPoints: &point atIndex: 1];
        }
        else
        {
            [rubberBand setAssociatedPoints: &point atIndex: 1];
        }
        
        lastPoint = point;
        //[controller updateMainTabView];
        [self setNeedsDisplay:YES];
    }
	if (![newPath valid])
	{
		if (displayMode == 0)
			[document removeCanvasObject: newPath];
		else if (displayMode == 1)
			[document removeNavigationObject: newPath];
		[controller updateMainTabView];
	}
	else
	{
		[newPath adaptPathToRects];
	}
	
    [rubberBand release];
	rubberBand = nil;
	focusRingObject = nil;
	newPath = nil;
	[nc postNotificationName: @"trailCreated" object: nil];  
	[self setNeedsDisplay:YES];
}

- (void)mouseDown:(NSEvent *)theEvent // handels user input (main mouse button pressed)
{
    if (((displayMode == 0) || (displayMode == 1)) && (editMode == 0))
    {
		if ([theEvent clickCount]==2)
		{
			NSPoint point = [self convertPointToModel: [self convertPoint:[theEvent locationInWindow] fromView:nil]];
			
			OCEObject *object= [self objectUnderPoint:point];
			if (object)
				[object doubleKlickAtPoint:point];
			[self setNeedsDisplay:YES];
		}
		else
		{
			[self selectAndTrackMouseWithEvent:theEvent];
		}
	}
    else if (((displayMode == 0) || (displayMode == 1)) && (editMode == 1))
    {
        [self createPathWithEvent: theEvent];
    }
	else if (((displayMode == 0) || (displayMode == 1)) && (editMode == 3))
    {
        [self moveCanvasWithEvent: theEvent];
    }
	else if ((displayMode == 1) && (editMode == 2))
	{
		[self createFrameWithEvent: theEvent];
	
	}
}

- (void)mouseEntered:(NSEvent *)theEvent
{
	if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
		[[NSCursor openHandCursor] set];
}

- (void)mouseExited:(NSEvent *)theEvent
{
	if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
		[[NSCursor arrowCursor] set];
}

- (void)keyDown:(NSEvent *)theEvent
{
    switch ([theEvent keyCode])
    {
        case 49: // space
        {
            spaceIsPressed = YES;
			// dont use in pre10.3
			if (((displayMode == 1) || (displayMode == 0)) && (!editMode))
			{
				if (!trackingRect)
					trackingRect = [self addTrackingRect: [self bounds] owner: self userData: nil assumeInside: NO];
				if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
					[[NSCursor openHandCursor] set];
			}
			
            break; 
        }
        default:
        {
            [self interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
        }
    }
}

- (void)keyUp:(NSEvent *)theEvent
{
    if ([theEvent keyCode] == 49)
    {
        spaceIsPressed = NO;
		if (trackingRect)
		{
			[self removeTrackingRect:trackingRect];
			trackingRect = 0;
		}
		if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
			[[NSCursor arrowCursor] set];
    }
}

- (void)deleteForward:(id)sender
// wird beim drcken von entf ausgefhrt. hat ursprung in NSResponder
{
    if (((displayMode == 0) || (displayMode == 1)) && (!(editMode == 3))) {
        OCEObject *object;
		BOOL resetStartFrame = NO;
        int i, c = [selectedObjects count]-1;
        for (i = c; i > -1; i--)  {
            object = [selectedObjects objectAtIndex:i];
            if (displayMode) {
				bool flag = YES;
				if ([object isMemberOfClass:[OCENavFrameObject class]]) {
					flag = ([(OCENavFrameObject *) object deleteActiveComponent]) ? NO : YES;
					resetStartFrame = ([(OCENavFrameObject *) object isStartFrame]) ? YES : resetStartFrame;
				}
				
				if (flag) { // only delete if there was no activeComponent selected
					[document removeNavigationObject:object];
				}
			} else {
				[document removeCanvasObject:object];
			}
        }
		if (displayMode)
			[document removeUnassociatedNavigationTrails]; // trails die kein end oder start rect haben

		if (resetStartFrame)
			[document setNavigationStartFrame];
		
        [self setNeedsDisplay:YES];
        [selectedObjects removeAllObjects];
        [controller updateMainTabView];
    }
}

- (void)deleteBackward:(id)sender
{
    [self deleteForward:sender];
}

- (void) keyStrokeEffektingXBy: (float) x yBy: (float) y
{
	if (((displayMode == 0) || (displayMode == 1)) && (editMode == 0))
    {
        int i,c;
        c = [selectedObjects count];
        if (c) // there are objects selected
        {
			//NSPoint origin, oldOrigin;
			OCEObject *object;
			float xOffset, yOffset;
            for (i=0; i<c; i++) 
            {
				xOffset = x;
				yOffset = y;
				object = [selectedObjects objectAtIndex:i];
				[object startManipulationWithSelector:@selector(moveXBy:moveYBy:) andData: nil];
				if (([document snapToGrid]) && ([object snapAble]))
				{
					[object gridObject:[document gridSize]];
					xOffset *= [document gridSize];
					yOffset *= [document gridSize];					
				}
                [object moveXBy: xOffset moveYBy: yOffset];
            }
            
            // register undo
            [selectedObjects makeObjectsPerformSelector:@selector(stopManipulation)];
            [[document undoManager] setActionName:@"Move"];
        }
        else
        {
			canvasRect.origin.x = canvasRect.origin.x + x * 10;
            canvasRect.origin.y = canvasRect.origin.y + y * 10;
        }
    }
    
    [self setNeedsDisplay:YES];
    [controller updateMainTabView];
}

- (void)moveDown:(id)sender
{
	[self keyStrokeEffektingXBy: 0 yBy: -1];
}

-(void) moveDownAndModifySelection:(id)sender
{
	[self keyStrokeEffektingXBy: 0 yBy: -10];
}

- (void)moveUp:(id)sender
{
	[self keyStrokeEffektingXBy: 0 yBy: 1];
}

-(void) moveUpAndModifySelection:(id)sender
{
	[self keyStrokeEffektingXBy: 0 yBy: 10];
}

- (void)moveLeft:(id)sender
{
	[self keyStrokeEffektingXBy: -1 yBy: 0];
}

-(void) moveLeftAndModifySelection:(id)sender
{
	[self keyStrokeEffektingXBy: -10 yBy: 0];
}

-(void) moveBackwardAndModifySelection:(id)sender
{
	// workaround for moveLeftAndModifySelection on pre10.3
	if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2)
	{
		[self keyStrokeEffektingXBy: -10 yBy: 0];
	}
}

- (void)moveRight:(id)sender
{
	[self keyStrokeEffektingXBy: 1 yBy: 0];
}

-(void) moveRightAndModifySelection:(id)sender
{
	[self keyStrokeEffektingXBy: 10 yBy: 0];
}

-(void) moveForwardAndModifySelection:(id)sender
{
	// workaround for moveRightAndModifySelection on pre10.3
	if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2)
	{
		[self keyStrokeEffektingXBy: 10 yBy: 0];
	}
}

// ----------------------------------------------------------------------
// UI-Menu Methoden
// ----------------------------------------------------------------------
// bei diesen methoden schummle ich etwas. die daten werden nicht auf das pasteboard geschrieben sondern lokal gespeichert
- (IBAction)copy:(id)sender
{
	if (displayMode != 2)
	{
		if ([selectedObjects count]) 
		{
			[fakePasteBoard removeAllObjects];
			NSEnumerator *objectsEnumerator = [selectedObjects objectEnumerator];
			OCEObject *selectedObject;
			
			while(selectedObject = [objectsEnumerator nextObject]) 
			{            
				[fakePasteBoard addObject:[selectedObject copy]];
			}
		}
	}
}

- (IBAction)cut:(id)sender 
{
    [self copy:sender];
    [self delete:sender];
}

- (IBAction)paste:(id)sender 
{
	if (displayMode != 2)
	{
		[selectedObjects removeAllObjects];
		
		if ([fakePasteBoard count]) 
		{
			int i = 0;
			int offset = 10;
			BOOL add = YES;
			
			NSPoint location = (NSPoint) {NSMidX(canvasRect),NSMidY(canvasRect)};
			
			if ([document snapToGrid]) {
				offset = [document gridSize];
				location = [self gridPoint:location];
			}
			
			NSEnumerator *objectsEnumerator = [fakePasteBoard objectEnumerator];
			OCEObject *object;
			
			while(object = [objectsEnumerator nextObject]) 
			{
				add = true;
				OCEObject *newCopy = [object copy];
				if (i)
				{
					location.x += offset;
					location.y -= offset;
				}
				i++;
				
				if ([newCopy isKindOfClass:[OCERectObject class]] )
				{
					[(OCERectObject *) newCopy setOriginX:location.x];
					[(OCERectObject *) newCopy setOriginY:location.y];
				}
				else if ( [newCopy isKindOfClass:[OCETrailObject class]] ) {
					NSPoint old = [(OCEPathObject *) newCopy startPoint];
					[newCopy moveXBy: location.x-old.x moveYBy: location.y-old.y];    
				} else if ( [newCopy isKindOfClass:[OCENavTrailObject class]] ) {
					add = NO;
				}
				
				if (displayMode == 0)
					[document addCanvasObject:newCopy];
				else if (displayMode == 1)
					[document addNavigationObject:newCopy];
				
				if (add)
					[selectedObjects addObject:newCopy];
			} // while
		} // fakePasteBoard count
		[controller updateMainTabView];
	}
}

- (IBAction)delete:(id)sender
{
    [self deleteForward: sender];
}

- (IBAction)selectAll:(id)sender
{
	if (displayMode != 2)
	{
		NSArray *objects = ((displayMode) ? [document navigationObjects] : [document canvasObjects]);
		
		[selectedObjects removeAllObjects];
		NSEnumerator *objectsEnumerator = [objects objectEnumerator];
		OCEObject *object;
		while(object = [objectsEnumerator nextObject]) 
		{
			[selectedObjects addObject: object];
		}
		[controller updateMainTabView];
		[self setNeedsDisplay:YES];	
	}
}

- (IBAction)selectVisible:(id)sender
{
	if (displayMode != 2)
	{
		[self clearSelection];
		NSArray *objects = ((displayMode) ? [document navigationObjects] : [document canvasObjects]);
		OCEObject *object;
		unsigned int i = 0;
		while (i < [objects count]) 
		{
			object = [objects objectAtIndex:i];
			if (NSIntersectsRect([object bounds],canvasRect))
			{
				[self selectObject:object];
			}
			i++;
		}
		[controller updateMainTabView];
		[self setNeedsDisplay:YES];	
	}
}

- (IBAction)selectNone:(id)sender
{
	[self clearSelection];
	[controller updateMainTabView];
	[self setNeedsDisplay:YES];	
}



@end
