//
//  OCEDocument.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEDocument.h"
#import "OCEObject.h"
#import "OCETrailObject.h"
#import "OCECanvas.h"
#import "OCEPanelObject.h"
#import "OCERectObject.h"
#import "OCENavFrameObject.h"
#import "OCENavTrailObject.h"
#import "OCEPanel.h"
#import "OCEPanel.h"
#import "OCEWindowController.h"
#import "OCEPanelView.h"

@implementation OCEDocument

// ----------------------------------------------------------------------
// Init und Controller Zeug
// ----------------------------------------------------------------------
- (id)init
{
    [super init];
    if (self) 
    {
        panels = [[NSMutableArray arrayWithCapacity:0] retain];
        canvasObjects = [[NSMutableArray arrayWithCapacity:0] retain];
        navigationObjects = [[NSMutableArray arrayWithCapacity:0] retain];
        nc = [NSNotificationCenter defaultCenter];
        panelIDCount=10;
        rectObjectIDCount=9;
        canvasColor = [[NSColor whiteColor] retain];
        undoManager = [self undoManager];
        
       /* arrowImagePath = [[[NSBundle mainBundle] pathForResource:@"arrowImage" ofType:@"gif"] retain];
        // load arrowImage
        arrowImage = [[NSImage alloc] initWithContentsOfFile: arrowImagePath];
        if (arrowImage) 
        {
            [arrowImage setScalesWhenResized:YES];
            NSSize imageSize = (NSSize) {24,24};
            [arrowImage setSize: imageSize];
        }*/
        
        // Navigation
        navigationTool = 1;
		navigationButtons = YES;
		activeComponentHint = YES;
		firstNavFrame = YES;
		
		// grid attributes
		gridSize = 15;
		showGrid = NSOnState;
		snapToGrid = NSOffState;
		gridColor = [[NSColor gridColor] retain];
		
		previewPath = nil;
    }
    return self;
}

- (void)makeWindowControllers
{
	/*if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_0) 
	{

	} 
	else if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) 
	{

	}
	else */
	{
		/* On a 10.2or later system */
		windowController = [[OCEWindowController alloc] initWithWindowNibName:@"OCEDocument"];
		[self addWindowController:windowController];

	}
}

-(NSString *)windowNibName
{
     return @"OCEDocument";
}

- (void)dealloc 
{
	[self removeWindowController:windowController];
    [panels release];
    [canvasObjects release];
    [canvasColor release];
    //[arrowImage release];
    //[arrowImagePath release];
    [gridColor release];

	if (previewPath != nil) {
		NSFileManager *manager = [NSFileManager defaultManager];
		[manager removeFileAtPath: previewPath handler: nil];
		[previewPath release];
	}
	
    [super dealloc];
}

- (OCEWindowController *) windowController;
{
	return windowController;
}

// ----------------------------------------------------------------------
// Methoden fr das Laden und Speichern
// ----------------------------------------------------------------------

- (NSFileWrapper *)fileWrapperRepresentationOfType:(NSString *)type
{
	NSMutableDictionary *rootDirDic = [NSMutableDictionary dictionary];
	NSFileWrapper *myComic = [[NSFileWrapper alloc] initRegularFileWithContents:[self dataRepresentationOfType:type]];
	[rootDirDic setObject:  [myComic autorelease] forKey: @"comic.xml"];
	return [[[NSFileWrapper alloc] initDirectoryWithFileWrappers:rootDirDic] autorelease];
}


- (BOOL)writeToFile:(NSString *)fullDocumentPath 
			 ofType:(NSString *)docType 
	   originalFile:(NSString *)fullOriginalDocumentPath 
	  saveOperation:(NSSaveOperationType)saveOperationType
{
	if (![self writeToFile:fullDocumentPath ofType:docType])
		return NO;
		
	NSFileManager *manager = [NSFileManager defaultManager];
	
	// create panels directory
	NSString *destination = [fullDocumentPath stringByAppendingPathComponent: @"/panels/"];
	fullOriginalDocumentPath = [fullOriginalDocumentPath stringByAppendingPathComponent: @"/panels/"];
	
	[manager createDirectoryAtPath: destination attributes: nil];
	
	// copy panels
	int i, c = [panels count];
	for (i=0; i<c; i++) {
		NSString *source = [(OCEPanel *)[panels objectAtIndex:i] path]; //here
		if ([manager fileExistsAtPath: source]) {
			if (![manager copyPath: source 
							toPath: [destination stringByAppendingPathComponent: [source lastPathComponent]]
						   handler: nil])
				return NO;
		}
		else { // from the old package
			source = [fullOriginalDocumentPath stringByAppendingPathComponent: [source lastPathComponent]];
			if ([manager fileExistsAtPath: source])
				if (![manager copyPath: source 
								toPath: [destination stringByAppendingPathComponent: [source lastPathComponent]]
							   handler: nil])
					return NO;
			
		}
	}
	
	return YES;
}

- (void) exportFlashTo: (NSString *) destinationPath;
{
	// show export progress sheet
	[windowController showExportProgressSheetStartValue: 1 endValue: [canvasObjects count] - 1];

	NSMutableString *data = [NSMutableString stringWithCapacity:0];	
	[data appendString: @"<ic version=\"1.3\">\n"];
	
	// canvas color
	int bgR, bgG, bgB;
	NSColor *color = [self canvasColor];
	if (![[color colorSpaceName] isEqualToString:@"NSCalibratedRGBColorSpace"])
		if ((color = [color colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"]) == nil)
			color = [[NSColor whiteColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];
	bgR = [color redComponent] * 255;
	bgG = [color greenComponent] * 255;
	bgB = [color blueComponent] * 255;
	[data appendFormat: @"<canvas color=\"0x%02x%02x%02x\">\n", bgR, bgG, bgB];
	
	// canvasObjects
	NSString *documentPath = [self fileName];
	if (documentPath != nil) {
		documentPath = [[documentPath stringByAppendingPathComponent:@"ic-resources"] stringByAppendingPathComponent: @"panels"];
	}
	unsigned i,c = [canvasObjects count];
	unsigned j = 0;
	for (i=0; i < c; i++) {
		if ([[canvasObjects objectAtIndex:i] isMemberOfClass: [OCEPanelObject class]]) {
			[data appendString: [[canvasObjects objectAtIndex:i] exportFlash: destinationPath documentPath: documentPath]];
			j++;
		} else {
			[data appendString: [[canvasObjects objectAtIndex:i] exportFlash]];
		}
		
		// increase export progress indicator
		[windowController setExportProgress];
	}
	[data appendString: @"</canvas>\n"];
		
	// navigation
	[data appendFormat:@"<navigation mode=\"%i\" firstframe=\"%i\" hotspothint=\"%@\" scale=\"false\">\n", navigationTool, [self startFrameID], (activeComponentHint ? @"true" : @"false")];
	c = [navigationObjects count];
	for (i=0; i<c; i++) {
		if ([[navigationObjects objectAtIndex:i] isMemberOfClass: [OCENavFrameObject class]]) {
			NSString *string = [[navigationObjects objectAtIndex:i] exportFlash];
			if (string != nil) {
				[data appendString: string];
			}
		}
	}
	
	[data appendString: @"</navigation>\n</ic>"];

		
	if (![data writeToFile: [destinationPath stringByAppendingPathComponent: @"comic.xml"] atomically:YES])
		NSLog(@"Couldn't write comic.xml for Flash export!");
		
	// close export progress sheet
	[windowController closeExportProgressSheet];
}




- (BOOL)loadFileWrapperRepresentation:(NSFileWrapper *)wrapper ofType:(NSString *)type
{
	if (![wrapper isDirectory])
		return NO;

	NSData *myData = [(NSFileWrapper *)[[wrapper fileWrappers] objectForKey: @"comic.xml"] regularFileContents];

	if (![self loadDataRepresentation: myData ofType: type])
		return NO;
	
	return YES;
}

- (NSData *)dataRepresentationOfType:(NSString *)type 
{
    if ([type isEqualToString:OCEDocumentType])
    {
		if ([windowController exportInProgress]) {
			NSLog(@"Do we really get here?");
			return nil;
			
		} else {
			NSArray *array = [self dataRepresentation];
			NSString *error;
			NSData *data = [NSPropertyListSerialization dataFromPropertyList: array 
															  format: NSPropertyListXMLFormat_v1_0 
													errorDescription: &error];
			if (data) {
				return data;
			} else {
				NSLog(error);
				[error release];
			}
		}
    }
    return nil;
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)type {
    if ([type isEqualToString:OCEDocumentType])
    {
    
        NSString *error;
        NSArray *array = [NSPropertyListSerialization propertyListFromData:data mutabilityOption:NSPropertyListImmutable 
                                                                              format:nil errorDescription: &error];
        if(array)
        {
			if ([(NSString *)[array objectAtIndex:0] compare: OCEDocumentVersion] == NSOrderedSame)
			{
				[self loadDocumentFromArray:array];
				[nc postNotificationName: @"canvasObjectsUpdated" object: nil];
				[[windowController panelView] setPanelCountDisplay];
				return YES;
			}
        }
        else
        {
            NSLog(error);
            [error release];
        }
    }
    return NO;
}



// ----------------------------------------------------------------------
// Methoden fr die Panels
// ----------------------------------------------------------------------
- (NSMutableArray *) dataRepresentation;
{
	NSMutableArray *array = [NSMutableArray arrayWithCapacity:16];
	
	// General
	[array addObject:OCEDocumentVersion];
	[array addObject:[NSNumber numberWithInt:[self gridSize]]];
	[array addObject:[NSNumber numberWithBool:[self showGrid]]];
	[array addObject:[NSNumber numberWithBool:[self snapToGrid]]];
	[array addObject:[NSNumber numberWithInt:panelIDCount]];
	[array addObject:[NSNumber numberWithInt:rectObjectIDCount]];
	[array addObject:[self arrowImagePath]];
	
	NSColor *tmpColor = [canvasColor colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
	if (tmpColor == nil) {
			NSLog(@"Couldn't convert color to RGB color space!");
			tmpColor = [[NSColor whiteColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];
	}
	[array addObject:[NSNumber numberWithFloat:[tmpColor redComponent]]];
	[array addObject:[NSNumber numberWithFloat:[tmpColor greenComponent]]];
	[array addObject:[NSNumber numberWithFloat:[tmpColor blueComponent]]];
	
	// Navigation
	[array addObject:[NSNumber numberWithInt: navigationTool]];
	[array addObject:[NSNumber numberWithBool: navigationButtons]];
	[array addObject:[NSNumber numberWithBool: firstNavFrame]];
	[array addObject:[NSNumber numberWithBool: activeComponentHint]];

	// panels
	unsigned i, c = [panels count];
	NSMutableArray *objectArray = [NSMutableArray arrayWithCapacity:c];
	for (i=0; i<c; i++) 
		[objectArray addObject:[[panels objectAtIndex:i] dataRepresentation]];
	[array addObject: objectArray];
	
	
	// canvasObjects
	c = [canvasObjects count];
	objectArray = [NSMutableArray arrayWithCapacity:c];
	for (i=0; i<c; i++) 
		[objectArray addObject:[[canvasObjects objectAtIndex:i] dataRepresentation]];
	[array addObject:objectArray];
	
	// navigationObjects
	c = [navigationObjects count];
	objectArray = [NSMutableArray arrayWithCapacity:c];
	for (i=0; i<c; i++) 
		[objectArray addObject:[[navigationObjects objectAtIndex:i] dataRepresentation]];
	[array addObject:objectArray];
	
	return array;
}


- (void) loadDocumentFromArray:(NSArray *) array
{
    //[self init];
	NSString *filename = [self fileName];
	bool error = NO;

	
	// general
	[self setGridSize: [[array objectAtIndex:1] intValue]];
	[self setShowGrid: [[array objectAtIndex:2] intValue]];
	[self setSnapToGrid: [[array objectAtIndex:3] boolValue]];
	panelIDCount = [[array objectAtIndex:4] intValue];
	rectObjectIDCount = [[array objectAtIndex:5] intValue];
	
	
	// trail arrow image
	/*if (arrowImagePath)
		[arrowImagePath release];
	arrowImagePath = [[[filename stringByAppendingPathComponent: @"/images/"] 
										stringByAppendingPathComponent: [array objectAtIndex:6]] retain];*/
	
	canvasColor = [[NSColor colorWithCalibratedRed: [[array objectAtIndex:7] floatValue]
											 green: [[array objectAtIndex:8] floatValue]
											  blue: [[array objectAtIndex:9] floatValue]
											 alpha: 1.0] retain];
	// Navigation	
	navigationTool = [[array objectAtIndex:10] intValue];
	navigationButtons = [[array objectAtIndex:11] boolValue];
	firstNavFrame = [[array objectAtIndex:12] boolValue];
	activeComponentHint = [[array objectAtIndex:13] boolValue];

		
	// panels
	NSArray *panelArray = [array objectAtIndex:14];
	unsigned i, c = [panelArray count];
	[panels release];
	panels = [[NSMutableArray arrayWithCapacity:c] retain];
	NSString *path = [filename stringByAppendingPathComponent: @"/panels/"];
	
	for (i=0; i<c; i++) 
	{
		OCEPanel *panel = [OCEPanel panelFromArray: [panelArray objectAtIndex:i] inDirectory:path];
		if (panel)
			[panels addObject:panel];
		else 
			error = YES;
	}
	if (error)
	{
		NSRunAlertPanel(@"Could Not Load Image(s)", 
						@"Please make sure the images are located in the same directory as the Infinite Canvas document.", 
						@"OK", nil,nil);
		return;
	}
	
	// canvasObjects
	NSArray *objectsArray = [array objectAtIndex:15];
	i = [objectsArray count];
	c = i;
	[canvasObjects release];
	
	canvasObjects = [[NSMutableArray arrayWithCapacity:c] retain];
	for (i=0; i<c; i++) 
	{
		OCEObject *object = [OCEObject objectFromArray: [objectsArray objectAtIndex:i] forDocument: self];
		if (object)
		{
			[object setUndoManager:[self undoManager]];
			[canvasObjects addObject:object];
		}
	}
	
	NSEnumerator *objectsEnumerator = [canvasObjects objectEnumerator];
	OCEObject *object;
	while (object = [objectsEnumerator nextObject]) 
	{
		if ([object isKindOfClass: [OCEPathObject class]])
		{
			[(OCEPathObject *) object finishLoadingObject: self];
		}
	}
	//[self setArrowImageWithPath:arrowImagePath];
	
	// navigationObjects
	objectsArray = [array objectAtIndex:16];
	i = [objectsArray count];
	c = i;
	[navigationObjects release];
	
	navigationObjects = [[NSMutableArray arrayWithCapacity:c] retain];
	for (i=0; i<c; i++) 
	{
		OCEObject *object = [OCEObject objectFromArray: [objectsArray objectAtIndex:i] forDocument: self];
		if (object)
		{
			[object setUndoManager:[self undoManager]];
			[navigationObjects addObject:object];
		}
	}
	
	objectsEnumerator = [navigationObjects objectEnumerator];
	while (object = [objectsEnumerator nextObject]) 
	{
		if ([object isKindOfClass: [OCEPathObject class]])
		{
			[(OCEPathObject *) object finishLoadingObject: self];
		}
	}
	
	[[windowController panelView] setPanelCountDisplay];	
	[[windowController panelView] setToolTips];
    [[self undoManager] removeAllActions];
}

// ----------------------------------------------------------------------
// Methoden fr die Panels
// ----------------------------------------------------------------------
- (NSMutableArray *)getPanels
{
    return panels;
}

- (int)panelCount
{
    return [panels count];
}

- (void)setPanels:(NSArray *)newPanels {
    unsigned i = [panels count];
    while (i-- > 0) 
    {
        [self removePanelAtIndex:i];
    }
    i = [newPanels count];
    while (i-- > 0) 
    {
        [self insertPanel:[newPanels objectAtIndex:i] atIndex:0];
    }
}

- (bool)loadImagesFromPaths:(NSArray *)fileNames 
{
    NSEnumerator *pathEnumerator = [fileNames objectEnumerator];
    NSString *imagePath;
    bool returnValue = YES; 
	
	[[windowController panelView] clearSelection];
    
    while(imagePath = [pathEnumerator nextObject]) 
    {
        OCEPanel *panel;
        if (panel = [self panelFromPath: imagePath])
        { // panel wurde bereits geladen. image wird neu geladen
					NSLog(@"Panel already loaded. Trying to reload panel (%@)!",imagePath);
            [panel reloadImage];
        }
        else
        { // panel wird geladen
					NSFileManager *fileManager = [NSFileManager defaultManager];
					imagePath = [imagePath stringByExpandingTildeInPath];
					BOOL isDirectory = NO;
					if (![fileManager fileExistsAtPath:imagePath isDirectory:&isDirectory]) {
						NSLog(@"There is no file at path: %@!", imagePath);
						returnValue = NO;
					} else if (isDirectory) {
						NSLog(@"Supplied path is a directory: %@!", imagePath);
						returnValue = NO;
					} else if (![fileManager isReadableFileAtPath:imagePath]) {
						NSLog(@"Supplied path is not readable: %@!", imagePath);
						returnValue = NO;
					} else {
					
					
            OCEPanel *newImage = [[OCEPanel alloc] initWithContentsOfFile: imagePath];
            if (newImage) 
            {
                // zum array hinzufuegen
                [self insertPanel: newImage atIndex: [panels count]];
                [newImage setPanelID:panelIDCount];
                panelIDCount++;
				
				// don't select new importet panels in panelView - off since 1.0.1
				// [[windowController panelView] selectObject:newImage];
            
                // wo im panelView zeichnen
                NSSize previewImageSize = [newImage panelViewSize];
                NSSize imageSize = [[newImage image] size];
            
                int reihen = ((int) ceil((double) [panels count]/2.0) );
                reihen--;
                int xPos;
                if (fmod((double) [panels count],2.0) != 0)
                    xPos = cPreviewImageSpacing; // erste spalte
                else
                    xPos = cPreviewImageSpacing+cPreviewImageWidth+cPreviewImageSpacing; // zweite spalte
            
                int yPos = (cPreviewImageSpacing) + reihen*(cPreviewImageWidth+cPreviewImageSpacing);

                if (imageSize.width > imageSize.height) 
                {
                    yPos = yPos + (cPreviewImageHeight - previewImageSize.height) / 2;
                }
                else 
                {
                    if (imageSize.width < imageSize.height) 
                    {
                        xPos = xPos + (cPreviewImageWidth - previewImageSize.width) / 2;
                    }
                    else 
                    {
                        // gleiche groesse -> bereits beim initialisierung gesetzt
                    }
                }
        
                [newImage setPanelViewRect:NSMakeRect(xPos,yPos,previewImageSize.width,previewImageSize.height)];

                [newImage release];
            }
            else 
            {
                NSLog(@"Could not load Image: %@",imagePath);
                returnValue = NO;
            }
					}
        }
    }
    
    return returnValue;
}

- (void)insertPanel:(OCEPanel *)newPanel atIndex:(int)index
{
    // WARNUNG: ICH GEHE DAVON AUS, DASS PANEL VOLLSTNDIG INITIALISIERT WURDE 
    // UND WEISS WO ES GEZEICHNET WIRD (AUCH IM UNDO FALL)!!!!!!!!
	if ((![undoManager isUndoing]) || (![undoManager isRedoing]))
		[undoManager setActionName: @"Load Panel"];
    [[undoManager prepareWithInvocationTarget:self] removePanelAtIndex:index];
    [[undoManager prepareWithInvocationTarget:nc] postNotificationName: @"panelsUpdated" object: nil];

    OCEPanel *panel = nil;
    
    unsigned int i = index;
    while (i < [panels count]) 
    {
        panel = [panels objectAtIndex:i];
        NSRect rect = [panel panelViewRect];
        if ((rect.origin.x >= cPreviewImageSpacing) && (rect.origin.x<=cPreviewImageSpacing+cPreviewImageWidth))
        {
            rect.origin.x = rect.origin.x + (cPreviewImageWidth+cPreviewImageSpacing);
        }
        else
        {
            rect.origin.x = rect.origin.x - (cPreviewImageWidth+cPreviewImageSpacing);
            rect.origin.y = rect.origin.y + (cPreviewImageHeight+cPreviewImageSpacing);
        }
        [panel setPanelViewRect:rect];
        i++;
    }

    [panels insertObject:newPanel atIndex:index];
}

- (void)removePanel:(OCEPanel *)oldPanel
{
    //[self removeCanvasObject:oldPanel]; // auch von Canvas lschen ->FALSCH!!!!!!!!!
    
    NSEnumerator *panelsEnumerator = [panels objectEnumerator];
    OCEPanel *panel;
    unsigned index = 0;
    
    while(panel = [panelsEnumerator nextObject]) 
    {
        if (panel==oldPanel)
            break;        
        else
            index++;
    }
    [self removePanelAtIndex:index];
}

- (void)removePanelAtIndex:(int)index {
    id oldPanel = [[panels objectAtIndex:index] retain];
    OCEPanel *panel = nil;
    
    unsigned i = index;
    while (i++ < [panels count]-1) 
    {
        panel = [panels objectAtIndex:i];
        NSRect rect = [panel panelViewRect];
        if ((rect.origin.x >= cPreviewImageSpacing) && (rect.origin.x<=cPreviewImageSpacing+cPreviewImageWidth))
        {
            rect.origin.x = rect.origin.x + (cPreviewImageWidth+cPreviewImageSpacing);
            rect.origin.y = rect.origin.y - (cPreviewImageHeight+cPreviewImageSpacing);
        }
        else
        {
            rect.origin.x = rect.origin.x - (cPreviewImageWidth+cPreviewImageSpacing);
        }
        [panel setPanelViewRect:rect];
    }
    
    [panels removeObjectAtIndex:index];
    
	if ((![undoManager isUndoing]) || (![undoManager isRedoing]))
		[undoManager setActionName: @"Delete Panel"];
    [[undoManager prepareWithInvocationTarget:self] insertPanel:oldPanel atIndex:index];
    [[undoManager prepareWithInvocationTarget:nc] postNotificationName: @"panelsUpdated" object: nil];
    [oldPanel release];
}

- (OCEPanel *)panelFromID: (int) panelID 
{
    NSEnumerator *panelsEnumerator = [panels objectEnumerator];
    OCEPanel *panel;
    
    while(panel = [panelsEnumerator nextObject]) 
    {
        if ([panel panelID] == panelID)
            return panel;
    }
    return nil;
}

- (OCEPanel *) panelFromPath: (NSString *) filename
{
    NSString *path = [filename lastPathComponent];

    NSEnumerator *panelsEnumerator = [panels objectEnumerator];
    OCEPanel *panel;
    
    while(panel = [panelsEnumerator nextObject]) 
    {
	if ([[[panel path] lastPathComponent] isEqualToString: path])
            return panel;
    }
    return nil;
}

- (bool) panelExistsOnCanvas: (OCEPanel *) panel
{
	NSEnumerator *objectEnumerator = [canvasObjects objectEnumerator];
    OCEPanelObject *object;
    
    while(object = [objectEnumerator nextObject]) 
    {
		if (([object isKindOfClass: [OCEPanelObject class]]) && ([object panel] == panel))
			return YES;
    }
	return NO;
}

// ----------------------------------------------------------------------
// Methoden fr Canvas Objects
// ----------------------------------------------------------------------
- (NSMutableArray *)canvasObjects
{
    return canvasObjects;
}
- (void) addCanvasObject: (OCEObject *)object
{
    [self addCanvasObject:object atIndex:[canvasObjects count]];
}

- (void) addCanvasObject: (OCEObject *)object atIndex: (int)index
{	
    [canvasObjects insertObject:object atIndex:index];
	[object addedToCanvas: self];

    [[[self undoManager] prepareWithInvocationTarget:self] removeCanvasObjectAtIndex:index];
	if ((![undoManager isUndoing]) || (![undoManager isRedoing]))
		[undoManager setActionName: @"Add To Canvas"];
    [nc postNotificationName: @"canvasObjectsUpdated" object: nil];
}

- (void) removeCanvasObject: (OCEObject *)object
{
    NSEnumerator *objectsEnumerator = [canvasObjects objectEnumerator];
    OCEObject *aObject;
    unsigned index = 0;
    
    while(aObject = [objectsEnumerator nextObject]) 
    {
        if (aObject==object)
            break;        
        else
            index++;
    }
    [self removeCanvasObjectAtIndex:index];
}

- (void) removeCanvasObjectAtIndex: (int)index
{
    OCEObject *oldObject = [[canvasObjects objectAtIndex:index] retain];

    [canvasObjects removeObjectAtIndex:index];
    [[[self undoManager] prepareWithInvocationTarget:self] addCanvasObject:oldObject atIndex:index];
	
	if ((![undoManager isUndoing]) || (![undoManager isRedoing]))
		[undoManager setActionName: @"Delete From Canvas"];
        
    [nc postNotificationName: @"canvasObjectsUpdated" object: nil];
    [[[windowController canvas] selectedObjects] removeObject:oldObject];

	[oldObject removedFromCanvas:self];
    [oldObject release];
}

- (int) newRectObjectID;
{
    rectObjectIDCount++;
    return rectObjectIDCount;
}

- (OCERectObject *) rectObjectFromID: (int) rectID
{
    NSEnumerator *objectsEnumerator = [canvasObjects objectEnumerator];
    OCEObject *object;
    
    while (object = [objectsEnumerator nextObject]) 
        if ([object isKindOfClass: [OCERectObject class]])
            if (object = [(OCERectObject *) object rectObjectFromID:rectID])
                return (OCERectObject *) object;
	
	objectsEnumerator = [navigationObjects objectEnumerator];
    while (object = [objectsEnumerator nextObject]) 
        if ([object isKindOfClass: [OCERectObject class]])
            if (object = [(OCERectObject *) object rectObjectFromID:rectID])
                return (OCERectObject *) object;
	
    return nil;
}

// ----------------------------------------------------------------------
// Navigation Objects
// ----------------------------------------------------------------------
- (NSMutableArray *) navigationObjects
{
    return navigationObjects;
}

- (void) addNavigationObject: (OCEObject *)object
{
    [self addNavigationObject:object atIndex:[navigationObjects count]];
}

- (void) addNavigationObject: (OCEObject *)object atIndex: (int)index
{
	if ([object isKindOfClass:[OCENavFrameObject class]])
	{
		if (firstNavFrame)
		{
			[(OCENavFrameObject *)object setIsStartFrame:YES];
			firstNavFrame = NO;
		}
	}		
	
    [navigationObjects insertObject:object atIndex:index];
	[object addedToCanvas:self];
	
	
    [[[self undoManager] prepareWithInvocationTarget:self] removeNavigationObjectAtIndex:index];
	if ((![undoManager isUndoing]) || (![undoManager isRedoing]))
	{
		if ([object isMemberOfClass:[OCENavFrameObject class]] )
			[undoManager setActionName: @"Add Navigation Frame"];
        
		if ([object isMemberOfClass:[OCENavTrailObject class]] )        
			[undoManager setActionName: @"Add Navigation Trail"];
	}
    [nc postNotificationName: @"navigationObjectsUpdated" object: nil];
}

- (void) removeNavigationObject: (OCEObject *)object
{
    NSEnumerator *objectsEnumerator = [navigationObjects objectEnumerator];
    OCEObject *aObject;
    unsigned index = 0;
    
    while(aObject = [objectsEnumerator nextObject]) 
    {
        if (aObject==object)
            break;        
        else
            index++;
    }
    [self removeNavigationObjectAtIndex:index];
}

- (void) removeNavigationObjectAtIndex: (int)index
{
    OCEObject *oldObject = [[navigationObjects objectAtIndex:index] retain];

    [navigationObjects removeObjectAtIndex:index];
    [[[self undoManager] prepareWithInvocationTarget:self] addNavigationObject:oldObject atIndex:index];
	if ((![undoManager isUndoing]) || (![undoManager isRedoing]))
	{
		[undoManager setActionName: @"Delete Navigation Object"];
	}
    [nc postNotificationName: @"navigationObjectsUpdated" object: nil];
    [[[windowController canvas] selectedObjects] removeObject:oldObject];

	[oldObject removedFromCanvas:self];
    [oldObject release];
}

- (void) removeUnassociatedNavigationTrails
{
	unsigned i = [navigationObjects count];
    while (i-- > 0) 
    {
		OCEObject *object = [navigationObjects objectAtIndex:i];
		if ([object isMemberOfClass:[OCENavTrailObject class]] )        
			if (![(OCENavTrailObject *) object valid])
				[self removeNavigationObjectAtIndex: i];
    }
}

// ----------------------------------------------------------------------
// LAYER
// ----------------------------------------------------------------------
- (void) moveObjectUp:(OCEObject *)object
{
    if (object)
    { 
        [object retain];
      
		NSMutableArray *array;
		if ([[windowController canvas] displayMode] == 0)
			array = canvasObjects;
		else
			array = navigationObjects;
		
		unsigned index = [array indexOfObject:object];
		unsigned int count = [array count];
		
		if (count-1 != index) // auch damit nur eine undo operation registriert wird
		{
			
			unsigned index2 = index;
			index2++;
			
			while (index2<count)
			{
				if ([object intersectObject:[array objectAtIndex:index2]])
				{
					index2++;
					break;
				}
				else
					index2++;
			}
			[array insertObject:object atIndex:index2];
			[array removeObjectAtIndex:index];
			[nc postNotificationName: @"canvasObjectsUpdated" object: nil];
			
			[undoManager setActionName: @"Move Object Up"];
		}
        [object release];
    }
}

- (void) moveObjectTop:(OCEObject *)object
{
    if (object)
    { 
        [object retain];
		
		NSMutableArray *array;
		if ([[windowController canvas] displayMode] == 0)
			array = canvasObjects;
		else
			array = navigationObjects;
		
        unsigned index = [array indexOfObject:object];

        if ([array count]-1 != index) // auch damit nur eine undo operation registriert wird
        {
            [array removeObject:object];
            [array insertObject:object atIndex:[array count]];
			[nc postNotificationName: @"canvasObjectsUpdated" object: nil];

			[undoManager setActionName: @"Move Object To Top"];
        }
        [object release];
    }
}

- (void) moveObjectsTop:(NSArray *)objects
{
	NSEnumerator *objectsEnumerator = [(([[windowController canvas] displayMode] == 0) ? canvasObjects : navigationObjects) objectEnumerator];
    OCEObject *object;
	NSMutableArray *temp = [[NSMutableArray arrayWithCapacity:0] retain];

	// first sort array
    while (object = [objectsEnumerator nextObject]) 
    {
		if([objects indexOfObjectIdenticalTo: object]!=NSNotFound)
			[temp addObject:object];
    }
	objectsEnumerator = [temp objectEnumerator];
	while (object = [objectsEnumerator nextObject]) 
		[self moveObjectTop:object];
	
	[temp release];
}

- (void) moveObjectDown:(OCEObject *)object
{
    if (object)
    { 
        [object retain];
		
		NSMutableArray *array;
		if ([[windowController canvas] displayMode] == 0)
			array = canvasObjects;
		else
			array = navigationObjects;
		
        unsigned index = [array indexOfObject:object];
        
        if (0 != index) // auch damit nur eine undo operation registriert wird
        {
            unsigned index2 = index;
            
            if (index2)
                index2--;
        
            while (index2>0)
            {
                if ([object intersectObject:[array objectAtIndex:index2]])
                {
                    break;
                }
                else
                    index2--;
            }
        
            [array removeObjectAtIndex:index];        
            [array insertObject:object atIndex:index2];
			[nc postNotificationName: @"canvasObjectsUpdated" object: nil];

			[undoManager setActionName: @"Move Object Down"];
        }
        [object release];
    }
}

- (void) moveObjectBottom:(OCEObject *)object
{
    if (object)
    { 
        [object retain];
		
		NSMutableArray *array;
		if ([[windowController canvas] displayMode] == 0)
			array = canvasObjects;
		else
			array = navigationObjects;
		
        if (0 != [array indexOfObject:object]) // auch damit nur eine undo operation registriert wird
        {
            [array removeObject:object];
            [array insertObject:object atIndex:0];
			[nc postNotificationName: @"canvasObjectsUpdated" object: nil];
			[undoManager setActionName: @"Move Object To Bottom"];
        }
        [object release];
    }
}

- (void) moveObjectsBottom:(NSArray *)objects
{
	NSEnumerator *objectsEnumerator = [(([[windowController canvas] displayMode] == 0) ? canvasObjects : navigationObjects) objectEnumerator];
    OCEObject *object;
	NSMutableArray *temp = [[NSMutableArray arrayWithCapacity:0] retain];
	
	// first sort array
    while (object = [objectsEnumerator nextObject]) 
    {
		if([objects indexOfObjectIdenticalTo: object]!=NSNotFound)
			[temp addObject:object];
    }
	objectsEnumerator = [temp reverseObjectEnumerator];
	while (object = [objectsEnumerator nextObject]) 
		[self moveObjectBottom:object];
	
	[temp release];
}

// ----------------------------------------------------------------------
// CANVAS
// ----------------------------------------------------------------------
- (NSColor *) canvasColor {
    return canvasColor;
}

- (void) setCanvasColor: (NSColor *) newColor {
    if (canvasColor != newColor){
		[undoManager setActionName: @"Change Canvas Color"];
        [[undoManager prepareWithInvocationTarget:self] setCanvasColor:canvasColor];
        [newColor retain];
        [canvasColor release];
        canvasColor = newColor;
		[windowController setCanvasColor: canvasColor];
        [nc postNotificationName: @"canvasObjectsUpdated" object: nil];
    }
}

// ----------------------------------------------------------------------
// Trail
// ----------------------------------------------------------------------
- (NSImage *) arrowImage
{
    //return arrowImage;
	return nil;
}

- (NSString *) arrowImagePath
{
    //return arrowImagePath;
	return @"null";
}

/*- (void) setArrowImageWithPath: (NSString *) path;
{
    NSImage *newImage = [[NSImage alloc] initWithContentsOfFile: path];
    if (newImage) 
    {
		[undoManager setActionName: @"Change Arrow Image"];
        [[undoManager prepareWithInvocationTarget:self] setArrowImageWithPath: arrowImagePath];
    
        NSString *oldPath = arrowImagePath;
        arrowImagePath = [path retain];
        if (oldPath)
            [oldPath release];
		
		[newImage setScalesWhenResized:YES];
        NSSize imageSize = (NSSize) {24,24};
		[newImage setSize: imageSize];

        if (arrowImage)
            [arrowImage release];
		arrowImage = newImage;
                    
        [windowController setArrowImageInPreviews];

        NSEnumerator *objectsEnumerator = [canvasObjects objectEnumerator];
        OCEObject *object;
        while(object = [objectsEnumerator nextObject]) 
        {
            if ([object isMemberOfClass:[OCETrailObject class]])
            {
                [(OCETrailObject *) object setArrowImage:arrowImage];
            }
        }
        [[windowController canvas] setNeedsDisplay:YES];
    }
    
    if(!arrowImage)
    {
        NSRunAlertPanel(@"Failed To Load Arrow Image!", 
            @"The loaded document my be corrupted. I recommend that you quit this application and try to get help in the FAQs or from the maintainer of this application.", 
            @"Continue", nil,nil);
    }
}*/

// ----------------------------------------------------------------------
// Navigation
// ----------------------------------------------------------------------
- (void) setNavigationTool: (int) tool
{
	if (navigationTool != tool)
	{
		[undoManager setActionName: @"Change Navigation Tool"];
        [[undoManager prepareWithInvocationTarget:self] setNavigationTool:navigationTool];

		navigationTool = tool;
		[windowController setNavigationTools: navigationTool];

	}
}

- (int) navigationTool;
{
    return navigationTool;
}

- (void) setNavigationButtons: (bool) flag
{
}

- (bool) navigationButtons
{
	return navigationButtons;
}

- (void) setActiveComponentHint: (bool) flag;
{
	if (activeComponentHint != flag)
	{
		[undoManager setActionName: @"Active Component Hint Change"];
        [[undoManager prepareWithInvocationTarget:self] setActiveComponentHint:activeComponentHint];
		
		activeComponentHint = flag;
		[windowController setActiveComponentHint: activeComponentHint];
	}
}

- (bool) activeComponentHint
{
	return activeComponentHint;
}

- (void) resetNavigationStartFrame
{
	NSEnumerator *navigationObjectsEnumerator = [navigationObjects objectEnumerator];
    OCEObject *object;
    
    while (object = [navigationObjectsEnumerator nextObject]) 
    {
        if ([object isMemberOfClass: [OCENavFrameObject class]])
        {
            if([(OCENavFrameObject*) object isStartFrame])
			{
				[(OCENavFrameObject*) object setIsStartFrame:NO];
				return;       
			}
        }
    }
}

- (void) setNavigationStartFrame
{
	NSEnumerator *navigationObjectsEnumerator = [navigationObjects objectEnumerator];
    OCEObject *object;
	firstNavFrame = YES;
    
    while (object = [navigationObjectsEnumerator nextObject]) {
        if ([object isMemberOfClass: [OCENavFrameObject class]]) {
            [(OCENavFrameObject*) object setIsStartFrame:YES];
			firstNavFrame = NO;
			return;       
        }
    }	
}

- (OCENavFrameObject *) startFrame
{
    NSEnumerator *navigationObjectsEnumerator = [navigationObjects objectEnumerator];
    OCEObject *object;
    
    while (object = [navigationObjectsEnumerator nextObject]) 
    {
        if ([object isMemberOfClass: [OCENavFrameObject class]])
        {
            if([(OCENavFrameObject*) object isStartFrame])
               return (OCENavFrameObject*) object;       
        }
    }
    return nil;
}

- (int) startFrameID;
{
    NSEnumerator *navigationObjectsEnumerator = [navigationObjects objectEnumerator];
    OCEObject *object;
    
    while (object = [navigationObjectsEnumerator nextObject]) 
    {
        if ([object isMemberOfClass: [OCENavFrameObject class]])
        {
            if([(OCENavFrameObject*) object isStartFrame])
				return [(OCENavFrameObject*) object rectID];       
        }
    }
    return nil;
}
// ----------------------------------------------------------------------
// GRID
// ----------------------------------------------------------------------

- (int) gridSize 
{
    return gridSize;
}

- (void) setGridSize: (int) newSize 
{
    if (gridSize !=newSize)
    {
		[undoManager setActionName: @"Change Grid Size"];
        [[undoManager prepareWithInvocationTarget:self] setGridSize:gridSize];
        gridSize = newSize;
		[windowController setGridSize: gridSize];
        [[windowController canvas] setNeedsDisplay:YES];
    }
}

- (int) showGrid 
{
    return showGrid;
}

- (void) setShowGrid: (int) flag
{
    if (showGrid != flag) 
	{
		[undoManager setActionName: @"Show Grid Change"];
        [[undoManager prepareWithInvocationTarget:self] setShowGrid: showGrid];
		
        showGrid = flag;
		[windowController setShowGrid: showGrid];
        [[windowController canvas] setNeedsDisplay:YES];
    }
}

- (int) snapToGrid 
{
    return snapToGrid;
}

- (void) setSnapToGrid: (int) flag 
{
    if (snapToGrid != flag)
    {
		[undoManager setActionName: @"Snap To Grid Change"];
        [[undoManager prepareWithInvocationTarget:self] setSnapToGrid: snapToGrid];
		
        snapToGrid = flag;
		[windowController setSnapToGrid: snapToGrid];
    }
}

- (NSColor *) gridColor 
{
    return gridColor;
}

- (void) setGridColor: (NSColor *) newColor 
{
    if (gridColor != newColor)
    {
        [gridColor release];
        [newColor retain];
        gridColor = newColor;
        [[windowController canvas] setNeedsDisplay:YES];
    }
}


// ----------------------------------------------------------------------
// Other
// ----------------------------------------------------------------------
- (NSString *) previewPath {
	if (previewPath == nil){
		previewPath = [[[self fileName] lastPathComponent] stringByDeletingPathExtension];
		if (previewPath == nil) {
			previewPath = @"ic"; 
		}
		NSFileManager *manager = [NSFileManager defaultManager];
		[manager createDirectoryAtPath: @"/tmp/infinitecanvas.mindnode.com/"  attributes:nil];
		
		previewPath = [[NSString stringWithString:@"/tmp/infinitecanvas.mindnode.com/"] stringByAppendingString: previewPath];
		
		srandom(time(NULL));
		int i = random() % 100;
		previewPath = [[previewPath stringByAppendingFormat: @"%i", i] retain];
		[manager createDirectoryAtPath: previewPath  attributes:nil];
	} 
	return previewPath;
}

- (bool) autoGenerateNavigationFrameNavigation: (int) frameWidth;
{
	// this methode tries to generate a basic navigation based on the canvas objects
	int halfWidth = frameWidth / 2;

	NSEnumerator *canvasObjectsEnumerator = [canvasObjects objectEnumerator];
    OCEObject *object;

    while (object = [canvasObjectsEnumerator nextObject]) {
        if ([object isMemberOfClass: [OCEPanelObject class]]) {
			NSPoint point = [(OCEPanelObject *)object center];
			point.x -= halfWidth;
			point.y -= halfWidth;
			
			OCENavFrameObject *newFrame = [[OCENavFrameObject alloc] initWithRect: (NSRect) {point,{frameWidth,frameWidth}}];
			[self addNavigationObject:newFrame];
			[newFrame release];
			[newFrame setUndoManager:[self undoManager]];
			[newFrame setRotation: [(OCEPanelObject *) object rotation]];
        }
    }	

	return true;
}


@end
