//
//  OCEModeView.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEModeView.h"
#import "OCECanvas.h"


@implementation OCEModeView

- (id)initWithFrame:(NSRect)frameRect
{
    [super initWithFrame:frameRect];
    
    state = 0;
	disabled[0] = NO;
	disabled[1] = NO;
	disabled[2] = YES;
	disabled[3] = NO;

    selectRect=NSMakeRect(0.0,0.0,25.0,24.0);
    trailRect=NSMakeRect(25.0,0.0,25.0,24.0);
    rectRect=NSMakeRect(50.0,0.0,25.0,24.0);
    moveRect=NSMakeRect(75.0,0.0,25.0,24.0);
    
    modeImage = [NSImage imageNamed:@"modeButtonNormal.tiff"];
    modeImageSelected = [NSImage imageNamed:@"modeButtonSelected.tiff"];
	modeImageDisabled = [NSImage imageNamed:@"modeButtonDisabled.tiff"];
  
    if (!((modeImage)  && (modeImageSelected) && (modeImageDisabled)))
        NSRunAlertPanel(@"File Reading Error",@"Could not load Mode Images!", nil, nil, nil);
        
    nc = [NSNotificationCenter defaultCenter];
    [nc addObserver: self
           selector: @selector(objectCreated:)
               name: @"trailCreated"
             object: nil];
	
	[nc addObserver: self
           selector: @selector(objectCreated:)
               name: @"frameCreated"
             object: nil];

    onlyOneTime = NO;
    
    return self;
}

- (void)drawRect:(NSRect)rect
{
    [modeImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver];
    switch(state)
    {
        case 0: 
            [modeImageSelected compositeToPoint:selectRect.origin fromRect:selectRect operation:NSCompositeSourceOver];
            break;
        case 1: 
            [modeImageSelected compositeToPoint:trailRect.origin fromRect:trailRect operation:NSCompositeSourceOver];
            break;
        case 2: 
            [modeImageSelected compositeToPoint:rectRect.origin fromRect:rectRect operation:NSCompositeSourceOver];
            break;
        case 3: 
            [modeImageSelected compositeToPoint:moveRect.origin fromRect:moveRect operation:NSCompositeSourceOver];
            break;
    }
	
	if (disabled[0])
		[modeImageDisabled compositeToPoint:selectRect.origin fromRect:selectRect operation:NSCompositeSourceOver];
	if (disabled[1])
		[modeImageDisabled compositeToPoint:trailRect.origin fromRect:trailRect operation:NSCompositeSourceOver];
	if (disabled[2])
		[modeImageDisabled compositeToPoint:rectRect.origin fromRect:rectRect operation:NSCompositeSourceOver];
	if (disabled[3])
		[modeImageDisabled compositeToPoint:moveRect.origin fromRect:moveRect operation:NSCompositeSourceOver];

}


- (void)mouseDown:(NSEvent *)theEvent // handels user input (main mouse button released)
{
    int newState=0;
    NSPoint mouseLocation = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    
    if (NSMouseInRect(mouseLocation, selectRect, NO))
        newState=0;
        
    if (NSMouseInRect(mouseLocation, trailRect, NO))
    {
        newState=1;
        onlyOneTime = YES;
        if([theEvent clickCount] >= 2)
            onlyOneTime = NO;
    }   
    if (NSMouseInRect(mouseLocation, rectRect, NO))
	{
        newState=2;
        onlyOneTime = YES;
        if([theEvent clickCount] >= 2)
            onlyOneTime = NO;
	}
    if (NSMouseInRect(mouseLocation, moveRect, NO))
        newState=3;
	
	if (disabled[newState])
		return;
	
    [self setState:newState];
}


// i want to respond to events
- (BOOL)acceptsFirstResponder {
    return YES;
}

// wenn man auf den view klickt bewegt sich nicht das fenster mit
- (BOOL)mouseDownCanMoveWindow {
    return NO;
}

- (int)state {
    return state;
}

- (void)setState: (int) flag 
{
    if (state != flag)
    {
        state = flag;
        [self setNeedsDisplay:YES];
        [canvas setEditMode:state];
    }
}

- (void) objectCreated: (NSNotification *)notification
{
    if (onlyOneTime)
    {
        onlyOneTime = NO;
        [self setState:0];
    }
}

- (void)dealloc 
{
    [modeImage release];
    [modeImageSelected release];
	[modeImageDisabled release];

    [nc removeObserver: self];
    [super dealloc];
}

- (void) setDisplayMode: (int) flag
{
	if (flag == 0)
	{
		disabled[0]=NO;
		disabled[1]=NO;
		disabled[2]=YES;
		disabled[3]=NO;
		if (state == 2)
			state = 0;
	}
	else if (flag == 1)
	{			
		disabled[0]=NO;
		disabled[1]=NO;
		disabled[2]=NO;
		disabled[3]=NO;
	}			
	else 
	{
		disabled[0]=YES;
		disabled[1]=YES;
		disabled[2]=YES;
		disabled[3]=YES;
	}
	[self setNeedsDisplay:YES];
}


@end
