//
//  OCENavFrameObject.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCENavFrameObject.h"
#import "OCEActiveComponent.h"
#import "NSMutableArrayDeepCopy.h"
#import "OCEWindowController.h"
#import "OCECanvas.h"
#import "OCEDocument.h"
#import "OCEPathObject.h"

@implementation OCENavFrameObject

- (id)init
{
    [super init];
    
    isStartFrame = NO;
	waiting = YES;
	
	srandom(time(NULL));
	int i = random() % 7;
	switch (i)
	{
		case 0:
			borderColor = [[NSColor brownColor] retain];
			break;
		case 1:
			borderColor = [[NSColor orangeColor] retain];
			break;
		case 2:
			borderColor = [[NSColor yellowColor] retain];
			break;
		case 3:
			borderColor = [[NSColor greenColor] retain];
			break;
		case 4:
			borderColor = [[NSColor blueColor] retain];
			break;
		case 5:
			borderColor = [[NSColor purpleColor] retain];
			break;
		case 6:
			borderColor = [[NSColor grayColor] retain];
			break;
	}
	frameColor = [[borderColor colorWithAlphaComponent:0.25] retain];
	activeComponent = [[NSMutableArray arrayWithCapacity:0] retain];
	selectedComponent = nil;
	focusRingComponent = nil;
    
    [self setSnapAble:YES];
    isSquare = YES;
	
    [nc addObserver: self selector: @selector(didUndo:) name: @"NSUndoManagerDidRedoChangeNotification" object: nil];
    [nc addObserver: self selector: @selector(didUndo:) name: @"NSUndoManagerDidUndoChangeNotification" object: nil];	
    
	URL = @"";
	[URL retain];
	URLTarget = @"_self";
	[URLTarget retain];
	
	
	forceAspectRatio = YES;
	
    return self;
}

- (void)dealloc 
{
    [frameColor release];
    [borderColor release];
	[activeComponent release];
	[URL release];
	[URLTarget release];
	
	[super dealloc];
}

- (void) didUndo: (NSNotification *)notification
{
	selectedComponent = nil;
	focusRingComponent = nil;
}

// ----------------------------------------------------------------------
// Draw
// ----------------------------------------------------------------------
- (void) drawObjectIsSelected:(bool) selected
{
    [transform concat];
    [NSBezierPath setDefaultLineWidth:0.5];
    
	float drawWidth = ((width == 0) ? 1 : width);
	float drawHeight = ((height == 0) ? 1 : height);

    NSBezierPath *drawPath = [NSBezierPath bezierPathWithRect: NSMakeRect(-width/2.0,-height/2.0,drawWidth,drawHeight)];
	
	[frameColor set];
    [drawPath fill];
    [borderColor set];
    [drawPath stroke];
	
    if (selected)
        [self drawKnobs];
	
	// Draw Active Components
	NSAffineTransform *translate = [NSAffineTransform transform];
	[translate translateXBy: -width/2.0 yBy: -height/2.0];
	[translate concat]; 
	// damit das drawRect beim "origin" des Rects steht. so knnen sich die components richtig zeichnen
	
	//if (selected) {

	OCEActiveComponent *object;
	unsigned int i = 0;
	NSRect frameRect = NSMakeRect(0,0,width,height);
	while (i < [activeComponent count]) 
	{
		object = [activeComponent objectAtIndex:i];
		if (object == selectedComponent)
			[object drawObjectInRect: frameRect isSelected: YES];
		else
			[object drawObjectInRect: frameRect isSelected: NO];
		i++;
	}
	//}
	
	//if (activeComponent)
	//	[focusRingComponent drawFocusRing];
}

- (int) hitTest: (NSPoint)aPoint isSelected: (bool) isSelected
{
	int knob = [super hitTest: aPoint isSelected: isSelected];
	
	if (knob == Object) // check if component hit
	{
		OCEActiveComponent *object;
		int compKnob = 0;
		unsigned int i = 0;
		NSPoint point = [self convertPointToActiveComponent: aPoint];

		while (i < [activeComponent count]) 
		{
			object = [activeComponent objectAtIndex:i];
			if (compKnob = [object hitTest: point isSelected: (object == selectedComponent) ? YES : NO])
			{
				selectedComponent = object;
				return compKnob;
			}
			i++;
		}		
	}
	selectedComponent = nil;
	return knob;
}

- (enum corner) movingKnob: (enum corner) knob fromPoint: (NSPoint) lastPoint toPoint: (NSPoint) point 
				 withEvent: (NSEvent *) theEvent
{
	if (selectedComponent)
	{
		
		point = [self convertPointToActiveComponent: point];
		
		return [selectedComponent movingKnob: knob fromPoint: lastPoint toPoint: point withEvent:theEvent];
	}
	else
	{
		return [super movingKnob: knob fromPoint: lastPoint toPoint:  point withEvent: theEvent];
	}
}

- (void) moveXBy: (double) moveX moveYBy: (double) moveY;
{
	if (selectedComponent)
	{
		double theta = rotation * M_PI / 180;
		double cosTheta = cos(theta);
		double sinTheta = sin(theta);
		
		[selectedComponent moveXBy: (cosTheta * moveX + sinTheta * moveY) 
						   moveYBy: (-sinTheta * moveX + cosTheta * moveY)];
	}
	else
		[super moveXBy: moveX moveYBy: moveY];
}

- (NSPoint) pointForKnob: (enum corner) knob
{
	if (selectedComponent)
	{
		NSPoint point = NSZeroPoint;
		NSRect rect = [selectedComponent rect];
		// die middleknobs sind oft == 0 deshalb mssen sie nicht extra gesetzt werden
		
		if ((knob == LowerLeftKnob) || (knob == LowerMiddleKnob) || (knob == LowerRightKnob))
			point.y = -height/2.0 + rect.origin.y;
		
		if ((knob == UpperLeftKnob) || (knob == UpperMiddleKnob) || (knob == UpperRightKnob))
			point.y = -height/2.0 + rect.origin.y + rect.size.height;
		
		if ((knob == UpperLeftKnob) || (knob == MiddleLeftKnob) || (knob == LowerLeftKnob))
			point.x = -width/2.0 + rect.origin.x;
		
		if ((knob == UpperRightKnob) || (knob == MiddleRightKnob) || (knob == LowerRightKnob))
			point.x = -width/2.0 + rect.origin.x + rect.size.width;
		
		return [transform transformPoint: point];
		
	}
	else
	{
		return [super pointForKnob: knob];
	}
}

- (void) startManipulationWithSelector:(SEL)aSelector andData: (id) data;
{
	if (selectedComponent)
	{
		[selectedComponent startManipulationWithSelector: aSelector andData: data];
	}
	else
	{
		[super startManipulationWithSelector: aSelector andData: data]; 
		if ([NSStringFromSelector(manipulationSelector) isEqualToString: @"movingKnob:fromPoint:toPoint:withEvent:"])
		{
			unsigned int i = 0;
			while (i < [activeComponent count]) 
			{
				[[activeComponent objectAtIndex:i] startManipulationWithSelector: aSelector andData: data];
				i++;
			}
		}
	}
}

- (void) stopManipulation
{
	if (selectedComponent)
	{
		if (![selectedComponent stopManipulationInFrame: NSMakeRect(0,0,width,height)])
		{
			[manipulationData release];
			[self removeActiveComponent:selectedComponent];
			selectedComponent = nil;
		}
	}
	else
	{
		[super stopManipulation];
		[activeComponent makeObjectsPerformSelector: @selector(stopManipulation)];
		[self checkActiveComponent];
	}
}	

- (void) setUndoManager: (NSUndoManager *) newUndoManager
{
	[super setUndoManager:newUndoManager];
	[activeComponent makeObjectsPerformSelector: @selector(setUndoManager:) withObject: newUndoManager];

}

- (OCERectObject *) checkIfFocusRingObject: (NSPoint)point isStartPoint:(bool) startPoint
{
	OCERectObject *object = [super checkIfFocusRingObject:point isStartPoint: startPoint];
	
	if ((selectedComponent) && (startPoint))
	{
		focusRingComponent = selectedComponent;
		selectedComponent = nil;
		return focusRingComponent;
	}
	else
	{
		selectedComponent = nil;
		focusRingComponent = nil;
	}
	return object;
}

- (void) reset
{
	[super reset];
	[activeComponent makeObjectsPerformSelector: @selector(adaptPaths)];
}

- (void) removedFromCanvas: (OCEDocument *) document
{
	[super removedFromCanvas:document];
	[activeComponent makeObjectsPerformSelector: @selector(removedFromCanvas:) withObject: document];
}

- (void) gridObject: (int) gridSize
{
	[super gridObject: gridSize];
	unsigned int i = 0;
	while (i < [activeComponent count]) 
	{
		[[activeComponent objectAtIndex:i] gridObject: gridSize];
		i++;
	}
}

- (void) resizeTo: (NSRect) rect
{
	// alte werte speichern
	NSRect old = NSMakeRect(xPos,yPos,width,height);
	NSRect offset = NSZeroRect;
	
	[super resizeTo: rect];
	
	// berechnen um wieviel resized wurde
	if (rect.origin.x != old.origin.x)
	{
		offset.origin.x = (rect.origin.x-old.origin.x);
	}
	if (rect.origin.y == old.origin.y)
	{
		offset.origin.y = -rect.size.height+old.size.height;
	}
		
	
	unsigned int i = 0;
	while (i < [activeComponent count]) 
	{
		[[activeComponent objectAtIndex:i] adaptWithOffset: offset];
		i++;
	}
}

- (NSString *) URL {
	return URL;
}

- (NSString *) URLTarget {
	return URLTarget;
}

- (void) setURL: (NSString *) newURL withTarget: (NSString *) newTarget;
{
	[[undoManager prepareWithInvocationTarget:self] setURL: URL withTarget: URLTarget]; 
	[undoManager setActionName: @"Set URL"];

	id oldURL = URL;
	URL = [newURL retain];
	[oldURL release];
	
	id oldURLTarget = URLTarget;
	URLTarget = [newTarget retain];
	[oldURLTarget release];

}

// ----------------------------------------------------------------------
// Set Methods
// ----------------------------------------------------------------------
- (bool) isStartFrame;
{
    return isStartFrame;   
}

- (void) setIsStartFrame: (bool) flag
{
    if (isStartFrame != flag)
    {
        if (!editingObject)
        {
            [[undoManager prepareWithInvocationTarget:self] setIsStartFrame:isStartFrame]; 
			[undoManager setActionName: @"Change Start Frame"];
        }
        isStartFrame = flag;   
    }
}
- (bool) waiting
{
	return waiting;
}

- (void) setWaiting: (bool) flag
{
	if (flag != waiting)
	{
		[undoManager setActionName: @"Waiting For User Input Change"];
        [[undoManager prepareWithInvocationTarget:self] setWaiting:waiting];

		waiting = flag;
	}
}

- (NSColor *) frameColor
{
	return frameColor;
}

- (NSColor *) borderColor
{
	return borderColor;
}

- (bool) forceAspectRatio;
{
	return forceAspectRatio;
}

- (void) setForceAspectRatio: (bool) newValue {
	[undoManager setActionName: @"Force Aspect Ratio Change"];
	[[undoManager prepareWithInvocationTarget:self] setForceAspectRatio:forceAspectRatio];

	forceAspectRatio = newValue;
	isSquare = newValue;
	
	if (newValue) {
		if (width > height) {
			height = width;
		} else {
			width = height;
		}
		[self reset];
	}
}

// ----------------------------------------------------------------------
// Active Component Methods
// ----------------------------------------------------------------------
- (void) createActiveComponentForDocument: (OCEDocument *) document
{
	float compWidth = 30;
	float compHeight = 30;
	
	if ([document snapToGrid])
	{
		float gridSize = [document gridSize];
		float factor = 30 / gridSize;
		if (!factor)
			factor=1;
		compWidth = compHeight = gridSize * factor;
	}
	
	if (compWidth > width)
		compWidth = width;
	if (compHeight > height)
		compHeight = height;
	NSRect rect = NSMakeRect(0, height-compHeight,compWidth,compHeight);
	
	OCEActiveComponent *component = [[OCEActiveComponent alloc] initWithRect:rect];
	[component setColor:borderColor];
	[component setFrame:self];
	[component setUndoManager: undoManager];
	[component addedToCanvas:document];
	[self addActiveComponent: component];
	[component release];
}

- (void) addActiveComponent: (OCEActiveComponent *) component
{
	if ((![undoManager isUndoing]) && (![undoManager isRedoing]))
		[undoManager setActionName: @"Add Active Component"];
    [[undoManager prepareWithInvocationTarget:self] removeActiveComponent:component];
	[activeComponent addObject: component];
}

- (bool) deleteActiveComponent;
{
	if (selectedComponent)
	{
		[self removeActiveComponent: selectedComponent];
		selectedComponent = nil;
		return YES;
	}
	else
		return NO;
}


- (void) removeActiveComponent: (OCEActiveComponent *) component
{
	if ((![undoManager isUndoing]) && (![undoManager isRedoing]))
		[undoManager setActionName: @"Delete Active Component"];
    [[undoManager prepareWithInvocationTarget:self] addActiveComponent:component];
	[activeComponent removeObject: component];
	[component removedFromCanvas:nil];
}

- (NSPoint) convertPointToActiveComponent: (NSPoint) aPoint;
{
	NSAffineTransform *invertTrans = [transform copy];
    [invertTrans invert];
	
	aPoint = [invertTrans transformPoint:aPoint];
		
	aPoint.x +=  width/2.0;
	aPoint.y +=  height/2.0;
	
	return aPoint;
}

- (NSPoint) convertPointFromActiveComponent: (NSPoint) aPoint;
{
	aPoint.x -= width/2.0;
	aPoint.y -= height/2.0;
	return [transform transformPoint: aPoint];
}

- (void) checkActiveComponent
{
	OCEActiveComponent *object;
	int i = [activeComponent count]-1;
	NSRect frameRect = NSMakeRect(0,0,width,height);
	while (i >= 0) 
	{
		object = [activeComponent objectAtIndex:i];

		if ([object adaptToFrame:frameRect])
		{
			[object adaptPaths];
		}
		else
		{
			[object unregisterAssociatedPaths];
			[self removeActiveComponent:object];
			[undoManager setActionName: @"Resize"];
		}
		i--;
	}
}

- (void) addAssociatedPath: (OCEPathObject *) path // doesn't register the trail
{
	if (focusRingComponent)
	{
		[focusRingComponent addAssociatedPath:path];
		focusRingComponent = nil;
	}
	else
		[super addAssociatedPath:path];
}

- (void) removeAssociatedPath: (OCEPathObject *) path  // doesn't unregister the trail
{
	[super removeAssociatedPath:path];
	
}

- (void) transformForActiveComponent
{
	[transform concat];
	NSAffineTransform *translate = [NSAffineTransform transform];
	[translate translateXBy: -width/2.0 yBy: -height/2.0];
	[translate concat]; 	
}

- (OCERectObject *) activeComponentHitTest: (NSPoint) point
{
	OCEActiveComponent *object;
	unsigned int i = 0;
	
	while (i < [activeComponent count]) 
	{
		object = [activeComponent objectAtIndex:i];
		if ([object hitTest: point isSelected: (object == selectedComponent) ? YES : NO])
		{
			if ([[object outgoingAssociatedPaths] count])
				return object;
			else 
				return self;
		}
		i++;
	}
	
	return self;
}

- (bool) hasActiveComponent
{
	if ([activeComponent count])
		return YES;
	else
		return NO;
}

// ----------------------------------------------------------------------
// NSCopying Protocol
// ----------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
{
    OCENavFrameObject *copy = [super copyWithZone: zone];
    
    copy->isStartFrame = NO;
    copy->waiting = waiting;
	copy->borderColor = [borderColor retain];
	copy->frameColor = [frameColor retain];
	copy->activeComponent = [activeComponent deepCopy];
	copy->URL = [URL retain];
	copy->forceAspectRatio = forceAspectRatio;

	[copy->activeComponent makeObjectsPerformSelector: @selector(setFrame:) withObject:copy];
	
    return copy;
}

// ----------------------------------------------------------------------
// save stuff
// ----------------------------------------------------------------------
- (NSMutableArray *)dataRepresentation
{
	NSMutableArray *array = [[super dataRepresentation] retain];
	
	[array addObject:[NSNumber numberWithBool: isStartFrame]];
	[array addObject:[NSNumber numberWithBool: waiting]];

	NSColor *tmpColor = borderColor;
	if (![[tmpColor colorSpaceName] isEqualToString:@"NSCalibratedRGBColorSpace"])
		if ((tmpColor = [tmpColor colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"]) == nil)
			tmpColor = [[NSColor blackColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];
	
	[array addObject:[NSNumber numberWithFloat:[tmpColor redComponent]]];
	[array addObject:[NSNumber numberWithFloat:[tmpColor greenComponent]]];
	[array addObject:[NSNumber numberWithFloat:[tmpColor blueComponent]]];

	// activeComponent
	unsigned i, c = [activeComponent count];
	NSMutableArray *objectArray = [NSMutableArray arrayWithCapacity:c];
	for (i=0; i<c; i++) 
		[objectArray addObject:[[activeComponent objectAtIndex:i] dataRepresentation]];
	[array addObject: objectArray];
	
	[array addObject: URL]; // added in 1.3
	[array addObject: [NSNumber numberWithBool: forceAspectRatio]]; // added in 1.3

	return [array autorelease];
}

- (NSString *) exportFlash {
	NSMutableString *data = [NSMutableString stringWithFormat:@"<frame id=\"%i\" x=\"%.0f\" y=\"%.0f\" w=\"%.0f\" h=\"%.0f\" r=\"%.0f\">\n", rectID, xPos, yPos*-1, width, height, rotation];
	
	
	if ([URL length] > 0) { // no empty urls allowed
		[data appendFormat:@"<url link=\"%@\" target=\"%@\" />\n", URL, URLTarget];
	}
	
	if (!waiting) {
		// don't wait for user input
		[data appendString:@"<transition delay=\"0\" />\n"];
	}
	
	// activeComponent
	unsigned i, c = [activeComponent count];
	for (i=0; i<c; i++) {
		[data appendString: [[activeComponent objectAtIndex:i] exportFlash]];
	}
	
	// trails
	c = [associatedPaths count];
	for (i=0; i<c; i++) {
		if ([[[associatedPaths objectAtIndex:i] associatedStartRect] rectID] == rectID){
			[data appendString: [[associatedPaths objectAtIndex:i] exportFlash]];
		}
	}
	
	//[super exportFlash: data];
	[data appendString: @"</frame>\n"];
	return data;
}


- (id)initWithContentsOfArray:(NSArray *) array forDocument: (OCEDocument *) newDocument
{
    [self init];
	isStartFrame = [[array objectAtIndex:8] boolValue];
	waiting = [[array objectAtIndex:9] boolValue];
    
	[borderColor release];
    borderColor = [[NSColor colorWithCalibratedRed: [[array objectAtIndex:10] floatValue]
											 green: [[array objectAtIndex:11] floatValue]
											  blue: [[array objectAtIndex:12] floatValue] 
											 alpha: 1.0] retain];
	[frameColor release];
	frameColor = [[borderColor colorWithAlphaComponent:0.25] retain];
	
	//activeComponent
	NSArray *objectsArray = [array objectAtIndex:13];
	int c = [objectsArray count];
	int i = 0;
	
	[activeComponent release];
	activeComponent = [[NSMutableArray arrayWithCapacity:c] retain];
	for (i=0; i<c; i++) 
	{
		OCEObject *object = [OCEObject objectFromArray: [objectsArray objectAtIndex:i] forDocument: nil];
		if (object)
		{
			[object setUndoManager:[self undoManager]];
			[activeComponent addObject:object];
		}
	}
	[activeComponent makeObjectsPerformSelector: @selector(setColor:) withObject: frameColor];
	[activeComponent makeObjectsPerformSelector: @selector(setFrame:) withObject: self];
	
	if ([array count] > 15) { // new in 1.3
		//NSLog(@"testing: load URL");
		if ([(NSString *)[array objectAtIndex: 14] length] > 0)
			URL = [[array objectAtIndex: 14] retain];
		
		forceAspectRatio = [[array objectAtIndex: 15] boolValue];
		isSquare = forceAspectRatio;
	}
	
    return [super initWithContentsOfArray: array forDocument: newDocument];
}

// ----------------------------------------------------------------------
// ID Methoden
// ----------------------------------------------------------------------
- (OCERectObject *) rectObjectFromID: (int) anID;
{
	OCEActiveComponent *object;
	int i = 0;
	int c = [activeComponent count];
	
	while (i < c) 
	{
		object = [activeComponent objectAtIndex:i];
		if ([object rectObjectFromID: anID])
		{
			return object;
		}
		i++;
	}
	
	return [super rectObjectFromID:anID];
}

@end
