//
//  OCEObject.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEObject.h"
#import "OCECanvas.h"
#import "OCEDocument.h"
#import "NSBezierPathHitTest.h"

@implementation OCEObject

- (id)init 
{
    [super init];
    locked = NO;
    snapAble = NO;
    editingObject = NO;
	manipulationData = nil;
    undoManager = nil;
    nc = [NSNotificationCenter defaultCenter];
	
    return self;
}

- (void)dealloc 
{
	[nc removeObserver: self];
    [super dealloc];
}

// ----------------------------------------------------------------------
// Draw
// ----------------------------------------------------------------------
// eine dieser methoden MUSS in subclass berschrieben werden 
- (void) drawObjectIsSelected: (bool) selected
{
    [self drawObjectWithFraction: 1 isSelected:selected];
}

- (void) drawObjectWithFraction: (float) fraction isSelected: (bool) selected
{
    [self drawObjectIsSelected:selected];
}

// ----------------------------------------------------------------------
// Hit Tests and Knobs
// ----------------------------------------------------------------------
- (bool)intersectObject: (OCEObject *)object
{
	NSArray *returnArray = [[self path] intersectionsWithPath:[object path]];

    return ([returnArray count] > 1) ? YES : NO;
}
- (bool) intersectsRect: (NSRect) rect
{
    return NO;
}

- (int) hitTest: (NSPoint)aPoint isSelected: (bool) isSelected
{
    return NO; // muss von kindern implementiert werden!
}

- (enum corner) movingKnob: (enum corner) knob fromPoint: (NSPoint) lastPoint toPoint: (NSPoint) point 
				 withEvent: (NSEvent *) theEvent
{
    return knob;
}

+ (int)flipKnob:(int)knob horizontal:(BOOL)horizFlag {
    static BOOL initedFlips = NO;
    static int horizFlips[9];
    static int vertFlips[9];
    
    if (!initedFlips) {
        horizFlips[UpperLeftKnob] = UpperRightKnob;
        horizFlips[UpperMiddleKnob] = UpperMiddleKnob;
        horizFlips[UpperRightKnob] = UpperLeftKnob;
        horizFlips[MiddleLeftKnob] = MiddleRightKnob;
        horizFlips[MiddleRightKnob] = MiddleLeftKnob;
        horizFlips[LowerLeftKnob] = LowerRightKnob;
        horizFlips[LowerMiddleKnob] = LowerMiddleKnob;
        horizFlips[LowerRightKnob] = LowerLeftKnob;
        
        vertFlips[UpperLeftKnob] = LowerLeftKnob;
        vertFlips[UpperMiddleKnob] = LowerMiddleKnob;
        vertFlips[UpperRightKnob] = LowerRightKnob;
        vertFlips[MiddleLeftKnob] = MiddleLeftKnob;
        vertFlips[MiddleRightKnob] = MiddleRightKnob;
        vertFlips[LowerLeftKnob] = UpperLeftKnob;
        vertFlips[LowerMiddleKnob] = UpperMiddleKnob;
        vertFlips[LowerRightKnob] = UpperRightKnob;
        initedFlips = YES;
    }
    if (horizFlag) {
        return horizFlips[knob];
    } else {
        return vertFlips[knob];
    }
}

- (void) doubleKlickAtPoint: (NSPoint) aPoint
{
	
}

// ----------------------------------------------------------------------
// Attributes
// ----------------------------------------------------------------------
- (bool) locked
{
    return locked;
}

- (void) setLocked:(bool) flag
{
	[undoManager setActionName: @"Lock Position Change"];
    [[undoManager prepareWithInvocationTarget:self] setLocked:locked];
    locked = flag;
}

- (bool) snapAble
{
    return snapAble;   
}

- (void) setSnapAble: (bool) flag
{
    snapAble = flag;
}

- (bool) editingObject
{
    return editingObject;
}

- (void) startManipulationWithSelector:(SEL)aSelector andData: (id) data
{
	manipulationSelector = aSelector;
	manipulationData = [data retain];
    editingObject = YES;
	//[undoManager beginUndoGrouping];
}

- (void) stopManipulation
{
	if (!editingObject)
		return;
	[manipulationData release];
	manipulationSelector = nil;
    editingObject = NO;    
	//[undoManager endUndoGrouping];
}

- (void) moveXBy: (double) moveX moveYBy: (double) moveY
{
    
}

- (NSPoint) pointForKnob: (enum corner) knob
{
	return NSZeroPoint;
}

- (NSPoint) origin
{
    return NSZeroPoint;   
}

- (NSUndoManager *) undoManager
{
    return undoManager;
}

- (void) setUndoManager: (NSUndoManager *) newUndoManager
{
    undoManager = newUndoManager;
}

- (NSRect) bounds
{
    return bounds;   
}

- (void) addedToCanvas: (OCEDocument *) document
{
	
}

- (void) removedFromCanvas: (OCEDocument *) document
{
	
}

- (NSBezierPath *) path
{
	return nil;
}

- (void) gridObject: (int) gridSize
{
}

+ (NSPoint) gridPoint: (NSPoint) aPoint withGridSize: (int) gridSize
{	
    NSPoint bPoint = aPoint;
    bPoint.x = floor((bPoint.x / gridSize) + 0.5) * gridSize;
    bPoint.y = floor((bPoint.y / gridSize) + 0.5) * gridSize;
	
    
    int xFaktor = floor(aPoint.x / gridSize);
    float xOffset = aPoint.x - (gridSize * xFaktor);
	
    if (xOffset <= (gridSize/2.0))
        aPoint.x = xFaktor*gridSize;
    else
        aPoint.x = (xFaktor+1)*gridSize;
	
    int yFaktor = floor(aPoint.y / gridSize);
    float yOffset = aPoint.y - (gridSize * yFaktor);
	
    if (yOffset <= (gridSize/2.0))
        aPoint.y = yFaktor*gridSize;
    else
        aPoint.y = (yFaktor+1)*gridSize;
    
    return aPoint;
	
}

// ----------------------------------------------------------------------
// NSCopying Protocol
// ----------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
{
    OCEObject *copy = [[[self class] allocWithZone: zone] init];
    
    copy->locked = locked;
    copy->snapAble = snapAble;
    copy->bounds = bounds;
    [copy setUndoManager:[self undoManager]];
    
    return copy;
}


// ----------------------------------------------------------------------
// Save and Load
// ----------------------------------------------------------------------
- (NSMutableArray *)dataRepresentation
{
	NSMutableArray *array = [[NSMutableArray arrayWithCapacity:0] retain];
	
	[array addObject:NSStringFromClass([self class])];
	[array addObject:[NSNumber numberWithBool:locked]];
	
    return [array autorelease];
}

+ (id) objectFromArray: (NSArray *) array forDocument: (OCEDocument *) document
{
    Class theClass = NSClassFromString([array objectAtIndex:0]);
    
    return [[[theClass alloc] initWithContentsOfArray:array forDocument: document] autorelease];
}

- (id)initWithContentsOfArray:(NSArray *) array forDocument: (OCEDocument *) newDocument
{
	locked = [[array objectAtIndex:1] boolValue];
	
    return self;
}


- (NSString *) exportFlash: (NSString *) flashPath documentPath: (NSString *) documentPath;
{
	return nil;
}

- (NSString *) exportFlash {
	return nil;
}

@end
