//
//  OCEPanel.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEPanel.h"


@implementation OCEPanel

- (id)init 
{
    [super init];
    image = nil;
    panelViewImage=nil;
    panelViewSize = NSMakeSize(0,0);
    panelViewPoint = NSMakePoint(0,0);
    panelViewRect = NSMakeRect(0,0,0,0);
    panelID=0;
    path = nil;
	panelViewToolTip = nil;
        
    return self;
}

- (id)initWithContentsOfFile:(NSString *)newPath
{
    [self init];
    image = [[NSImage alloc] initWithContentsOfFile: newPath];
    if (image) {
        [image setFlipped:NO]; // das gleiche koordination system wie im view
        
        path = [newPath retain];
		panelViewToolTip = [[path lastPathComponent] retain];
        
        // size fr den panel view berechnen
        int previewWidth = cPreviewImageWidth;
        int previewHeight = cPreviewImageHeight;
        NSSize imageSize = [image size];
        if (imageSize.width > imageSize.height) 
        {
            float scale = (float) imageSize.width / (float) cPreviewImageWidth;
            previewHeight = (int) ((float)imageSize.height / scale);
        }
        else 
        {
            if (imageSize.width < imageSize.height) 
            {
                float scale = (float) imageSize.height / (float) cPreviewImageHeight;
                previewWidth = (int) ((float)imageSize.width / scale);
            }
            else 
            {
                // gleiche groesse -> bereits beim initialisierung gesetzt
            }
        }
        panelViewSize = NSMakeSize(previewWidth,previewHeight);
        
        panelViewImage=[image copy];
        
        [panelViewImage setScalesWhenResized:YES];
        [panelViewImage setSize: NSMakeSize(previewWidth-2,previewHeight-2)]; 
        [panelViewImage setFlipped:NO];
        return self;
    }
    else
    {
        [self autorelease];
        return nil;
    }
}

-(NSImage *)image
{
    return image;
}

-(NSImage *)panelViewImage
{
    return panelViewImage;
}

- (NSSize)panelViewSize
{
    return panelViewSize;
}

- (void)setPanelViewRect:(NSRect)newRect 
{
    panelViewRect = newRect;
    
    NSSize size = [self panelViewSize];
    NSPoint newPoint = NSMakePoint(newRect.origin.x+1,newRect.origin.y+size.height-1);
    [self setPanelViewPoint:newPoint]; 
}

- (NSRect)panelViewRect
{
    return panelViewRect;
}

- (void)setPanelViewPoint:(NSPoint)newPoint
{
    panelViewPoint = newPoint;
}

- (NSPoint)panelViewPoint
{
    return panelViewPoint;
}

- (int) panelID
{
    return panelID;
}

- (void) setPanelID: (int) newID
{
    panelID=newID;
}

- (NSString *) path
{
    return path;
}

- (NSString *) panelViewToolTip {
	return panelViewToolTip;
}

- (void) reloadImage
{
    NSImage *newImage = [[NSImage alloc] initWithContentsOfFile: path];
    if (newImage) {
        [image release];
        image=newImage;
        [newImage setFlipped:NO]; // das gleiche koordination system wie im view
        
        // size fr den panel view berechnen
        int previewWidth = cPreviewImageWidth;
        int previewHeight = cPreviewImageHeight;
        NSSize imageSize = [newImage size];
        if (imageSize.width > imageSize.height) 
        {
            float scale = (float) imageSize.width / (float) cPreviewImageWidth;
            previewHeight = (int) ((float)imageSize.height / scale);
        }
        else 
        {
            if (imageSize.width < imageSize.height) 
            {
                float scale = (float) imageSize.height / (float) cPreviewImageHeight;
                previewWidth = (int) ((float)imageSize.width / scale);
            }
            else 
            {
                // gleiche groesse -> bereits beim initialisierung gesetzt
            }
        }
        NSSize oldSize = panelViewSize;
        panelViewSize = NSMakeSize(previewWidth,previewHeight);
        
        [panelViewImage release];
        panelViewImage=[newImage copy];
        
        [panelViewImage setScalesWhenResized:YES];
        [panelViewImage setSize: NSMakeSize(previewWidth-2,previewHeight-2)]; 
        [panelViewImage setFlipped:NO];
        
        if ((oldSize.width != panelViewSize.width) || (oldSize.height != panelViewSize.height))
        {
            int xPos = panelViewRect.origin.x-(cPreviewImageWidth/2 - oldSize.width/2);
            int yPos = panelViewRect.origin.y-(cPreviewImageHeight/2 - oldSize.height/2);
        
            if (imageSize.width > imageSize.height) 
            {
                yPos = yPos + (cPreviewImageHeight - panelViewSize.height) / 2;
            }
            else 
            {
                if (imageSize.width < imageSize.height) 
                {
                    xPos = xPos + (cPreviewImageWidth - panelViewSize.width) / 2;
                }
                else 
                {
                    // gleiche groesse -> bereits beim initialisierung gesetzt
                }
            }
            [self setPanelViewRect:NSMakeRect(xPos,yPos,panelViewSize.width,panelViewSize.height)];
        }
    }
}

- (void)drawInPanelView: (NSRect) panelView isSelected: (bool) selected
{
    [panelViewImage compositeToPoint:[self panelViewPoint] operation:NSCompositeSourceOver];

    if (selected) 
    {
        [[NSColor redColor] set];
        NSFrameRectWithWidth([self panelViewRect], 1.0);
    }
}

- (NSMutableArray *)dataRepresentation
{
	NSMutableArray *array = [NSMutableArray arrayWithCapacity:5];
    
    [array addObject: NSStringFromRect(panelViewRect)];
    [array addObject: NSStringFromSize(panelViewSize)];
    [array addObject: NSStringFromPoint(panelViewPoint)];
    [array addObject: [NSNumber numberWithInt:panelID]];
    [array addObject: [path lastPathComponent]];
    
    return array;
}

+ (OCEPanel *) panelFromArray: (NSArray *) array inDirectory: (NSString *)directory
{
    OCEPanel *panel = [[OCEPanel alloc] autorelease];
    panel = [panel initWithContentsOfArray: array inDirectory: directory];

    return panel;
}

- (id)initWithContentsOfArray: (NSArray *) array inDirectory: (NSString *)directory;
{
    [self init];

    path=[[[directory stringByAppendingString:@"/"] stringByAppendingString:[array objectAtIndex:4]] retain]; 
	panelViewToolTip = [[path lastPathComponent] retain];
    image = [[NSImage alloc] initWithContentsOfFile: path];
    if (!image) 
    {
        NSLog(@"Failed to Load Image:");
        NSLog([directory stringByAppendingString:path]);
        return nil; // failed to load image
    }
    panelViewRect = NSRectFromString([array objectAtIndex:0]);
    panelViewSize = NSSizeFromString([array objectAtIndex:1]);
    panelViewPoint = NSPointFromString([array objectAtIndex:2]);
    panelID = [[array objectAtIndex:3] intValue];
    
    panelViewImage=[image copy];
    [panelViewImage setScalesWhenResized:YES];
    [panelViewImage setSize: NSMakeSize(panelViewSize.width-2,panelViewSize.height-2)]; 
    [panelViewImage setFlipped:NO];
    
    return self;
}

- (void)dealloc 
{
    [image release];
    if (path)
        [path release];
	
	if (panelViewToolTip) 
		[panelViewToolTip release];
   
    [super dealloc];
}

@end
