//
//  OCEPanelObject.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEPanelObject.h";
#import "OCEPanel.h";
#import "OCECanvas.h";
#import "OCEDocument.h";
#import "ICFlashConverter.h";

@implementation OCEPanelObject


- (id)init
{
    [super init];
    
    panel = nil;

    style = scale;
    transparency=1.0;
    borderColor=[[NSColor blackColor] retain];
    borderThickness = 0.0;
    
    [self setSnapAble:YES];

    return self;
}

// ----------------------------------------------------------------------
// Draw
// ----------------------------------------------------------------------
- (void) drawObjectWithFraction: (float) fraction isSelected: (bool) selected
{
	// fraction berechnen
	float newFraction = transparency * fraction;
	
    [transform concat];
    NSImage *image = [panel image];

    [NSBezierPath setDefaultLineJoinStyle:NSMiterLineJoinStyle]; 
    
	float drawWidth = ((width == 0) ? 1 : width);
	float drawHeight = ((height == 0) ? 1 : height);
    NSRect drawRect = NSMakeRect(-width/2.0,-height/2.0,drawWidth,drawHeight);
    
    NSSize imageSize = [image size];
    imageSize.width = floor(imageSize.width)-1; // -1 um weisse flchen (rundungsfehler) zwischen den panels zu vermeiden
    imageSize.height = floor(imageSize.height)-1;
    NSRect imageRect = NSMakeRect(0.0,0.0,imageSize.width,imageSize.height);
        
    switch (style)
    {
        case scale:
        {
            [image drawInRect: drawRect fromRect: imageRect operation:NSCompositeSourceOver fraction: newFraction];
            break;
        }
        case crop:
        {
            NSRect convertedDrawRect = NSMakeRect(0.0,0.0,drawRect.size.width,drawRect.size.height);
            NSRect newDrawRect = NSIntersectionRect(convertedDrawRect,imageRect);
            float y = drawRect.origin.y;
            if ((drawRect.size.height-imageSize.height)>0)
                y=drawRect.origin.y+drawRect.size.height-imageSize.height;
            if ((drawRect.size.height-imageSize.height)<0)
                
                newDrawRect.origin.y = newDrawRect.origin.y+imageSize.height-drawRect.size.height;
            [image drawInRect: NSMakeRect(drawRect.origin.x,y,newDrawRect.size.width,newDrawRect.size.height)
                     fromRect: newDrawRect
                    operation:NSCompositeSourceOver 
                     fraction:newFraction];
            break;
        }
        case tile:
        {
            float h = floor(drawRect.size.height);
            while (h > 0)
            {
                if (h-imageSize.height<0)
                {
                    imageRect.origin.y = imageSize.height-h;
                    imageRect.size.height = h;
                    h = 0; // exit while
                }
                else
                {
                    h = h-imageSize.height;
                }
                float y = drawRect.origin.y+h;
                
                float w = 0;
                imageRect.size.width = imageSize.width; // weil es in der while schleife berschrieben wird
                
                while (w < drawRect.size.width)
                {
                    float x = drawRect.origin.x+w;
                    if (drawRect.size.width-w < imageSize.width)
                    {
                        imageRect.size.width = drawRect.size.width-w;
                        w = drawRect.size.width+1; // exit while
                    }
                    else
                    {
                        w = w+imageSize.width;
                    }
                    
                    [image drawInRect: NSMakeRect(x,y,imageRect.size.width,imageRect.size.height)  
                             fromRect: imageRect
                            operation:NSCompositeSourceOver 
							 fraction:newFraction];
                }
            }
            break;
        }
    }
    
    if (borderThickness != 0.0) // weil 0.0 als linewidth 1.0 gezeichnet wird && bei scale wird mit rotiert
    {
        NSRect borderRect = drawRect;
        borderRect.origin.x -= borderThickness/2.0-1;
        borderRect.origin.y -= borderThickness/2.0-1;
        borderRect.size.width  += borderThickness-2;
        borderRect.size.height += borderThickness-2;
        
        NSBezierPath *border = [NSBezierPath bezierPathWithRect: borderRect];
		
		if (fraction == 1)
		{
			[borderColor set];
		}
		else
		{
			[[borderColor colorWithAlphaComponent:fraction] set];
		}

        [border setLineWidth:borderThickness];
        [border stroke];
    }
    
    if (selected)
        [self drawKnobs];
}


- (OCEPanel *)panel
{
    return panel;
}

- (void) setPanel:(OCEPanel *)newPanel
{
    id oldPanel = panel;
    panel = [newPanel retain];
	NSSize size = [[panel image] size];
	proportion = size.height/size.width;
    [oldPanel release];
	//[ICFlashConverter convertPanel: self withDocumentPath: [panel path] writeToPath: @"/Users/markusm/m.swf"];

}

- (void) resetSize;
{
	NSSize size = [[panel image] size];
	[self setWidth:size.width];
	[self setHeight:size.height];
    if (!(([undoManager isUndoing]) || ([undoManager isRedoing])))
        [undoManager setActionName: @"Set Original Size"];
}

- (enum Style) style
{
    return style;
}

- (void) setStyle: (enum Style) newStyle
{
    [[undoManager prepareWithInvocationTarget:self] setStyle:style];
	[undoManager setActionName: @"Change Panel Style"];

    style = newStyle;
}

- (float) transparency
{
    return transparency;
}

- (void) setTransparency: (float) newTransparency
{
    [[undoManager prepareWithInvocationTarget:self] setTransparency:transparency];
    if (!(([undoManager isUndoing]) || ([undoManager isRedoing])))
        [undoManager setActionName: @"Change Transparency"];

    transparency = newTransparency;
}

- (NSColor *) borderColor
{
    return borderColor;
}

- (void) setBorderColor:(NSColor *) newColor
{
    [[undoManager prepareWithInvocationTarget:self] setBorderColor:borderColor];
    if (!(([undoManager isUndoing]) || ([undoManager isRedoing])))
        [undoManager setActionName: @"Change Border Color"];
        
    id oldColor = borderColor;
    borderColor = [newColor retain];
    [oldColor release];
}

- (float) borderThickness
{
    return borderThickness;
}

- (void) setBorderThickness: (float) newThickness
{
    [[undoManager prepareWithInvocationTarget:self] setBorderThickness:borderThickness];
    if (!(([undoManager isUndoing]) || ([undoManager isRedoing])))
        [undoManager setActionName: @"Set Panel Thickness"];
        
    borderThickness = newThickness;
}


- (void)dealloc 
{
    [panel release];
    [borderColor release];

    [super dealloc];
}

// ----------------------------------------------------------------------
// SAVE/LOAD METHODEN
// ----------------------------------------------------------------------
- (NSMutableArray *)dataRepresentation
{
	NSMutableArray *array = [[super dataRepresentation] retain];
	
    [array addObject:[NSNumber numberWithInt:[panel panelID]]];
    [array addObject:[NSNumber numberWithInt:style]];
    [array addObject:[NSNumber numberWithFloat:transparency]];
	
	NSColor *tmpColor = borderColor;
	if (![[tmpColor colorSpaceName] isEqualToString:@"NSCalibratedRGBColorSpace"])
		if ((tmpColor = [tmpColor colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"]) == nil)
			tmpColor = [[NSColor blackColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];
	
    [array addObject:[NSNumber numberWithFloat:[tmpColor redComponent]]];
    [array addObject:[NSNumber numberWithFloat:[tmpColor greenComponent]]];
    [array addObject:[NSNumber numberWithFloat:[tmpColor blueComponent]]];
	
	[array addObject:[NSNumber numberWithFloat:borderThickness]];

	return [array autorelease];
}

- (id)initWithContentsOfArray:(NSArray *) array forDocument: (OCEDocument *) newDocument
{
    [self init];
    OCEPanel *newPanel = [newDocument panelFromID: [[array objectAtIndex:8] intValue]];
    if (!newPanel)
        return nil;
    [self setPanel:newPanel];
    
    style = [[array objectAtIndex:9] intValue];
    transparency = [[array objectAtIndex:10] floatValue];
	[borderColor release];
    borderColor = [[NSColor colorWithCalibratedRed: [[array objectAtIndex:11] floatValue]
                                            green: [[array objectAtIndex:12] floatValue]
                                            blue: [[array objectAtIndex:13] floatValue] 
                                            alpha: 1.0] retain];
    borderThickness = [[array objectAtIndex:14] floatValue];

    return [super initWithContentsOfArray: array forDocument: newDocument];
}


- (NSString *) exportFlash: (NSString *) flashPath documentPath: (NSString *) documentPath;
{
	flashPath = [flashPath stringByAppendingPathComponent: [[panel path] lastPathComponent]];
	flashPath = [[flashPath stringByDeletingPathExtension] stringByAppendingPathExtension: @"swf"];
	
	// create SWF
	[ICFlashConverter convertPanel: self withDocumentPath: documentPath writeToPath: flashPath];

	// create XML
	NSString *data = [NSString stringWithFormat:@"<panel file=\"%@\" x=\"%.0f\" y=\"%.0f\" w=\"%.0f\" h=\"%.0f\" r=\"%.0f\" t=\"%.0f\" />\n", [flashPath lastPathComponent],[self originX],[self originY] * -1, width, height, rotation, transparency*100];

	return data;
}

// ----------------------------------------------------------------------
// NSCopying Protocol
// ----------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
{
    OCEPanelObject *copy = [super copyWithZone: zone];
    
    copy->panel = [panel retain];
    copy->style = style;
    copy->transparency = transparency;
    copy->borderColor = [borderColor retain];
    copy->borderThickness = borderThickness;
    
    return copy;
}




@end
