//
//  OCETrailDashView.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCETrailDashView.h"
#import "OCEWindowController.h"


@implementation OCETrailDashView
- (id)initWithFrame:(NSRect)frameRect
{
    [super initWithFrame:frameRect];
    
    state = 0;
    dash0Rect=NSMakeRect(0,36,90,36);
    dash1Rect=NSMakeRect(91,36,90,36);
    dash2Rect=NSMakeRect(0,0,90,36);
    dash3Rect=NSMakeRect(91,0,90,36);
    color = [[NSColor blackColor] retain];
    thickness = 3;
    
    dashArray1[0] = 5;
    dashArray1[1] = 5;

    dashArray2[0] = 8;
    dashArray2[1] = 3;
    dashArray2[2] = 8;
            
    dashArray3[0] = 9;
    dashArray3[1] = 6;
    dashArray3[2] = 3;

    return self;
}

- (void)drawRect:(NSRect)rect
{
    [[NSColor whiteColor] set]; 
    NSRectFill(rect);
    [[NSColor selectedControlColor] set]; 
    switch (state)
    {
        case 0:
            NSRectFill(dash0Rect);
            break;
        case 1:
            NSRectFill(dash1Rect);
            break;
        case 2:
            NSRectFill(dash2Rect);
            break;
        case 3:
            NSRectFill(dash3Rect);
            break;
    }
    [[NSColor blackColor] set];
    [NSBezierPath setDefaultLineWidth:2];
    [NSBezierPath strokeRect:rect];
    [NSBezierPath strokeLineFromPoint: (NSPoint) {90,0} toPoint: (NSPoint) {90,72}];
    [NSBezierPath strokeLineFromPoint: (NSPoint) {0,36} toPoint: (NSPoint) {180,36}];

    [color set];

    // state 0
    NSBezierPath *path = [NSBezierPath bezierPath];
    [path moveToPoint: (NSPoint) {5,54}];
    [path lineToPoint: (NSPoint) {85,54}];    
    [path setLineWidth:thickness];
    [path stroke];

    // state 1
    path = [NSBezierPath bezierPath];
    [path moveToPoint: (NSPoint) {95,54}];
    [path lineToPoint: (NSPoint) {175,54}];    
    [path setLineWidth:thickness];
    [path setLineDash:dashArray1 count:2 phase: 0.0];
    [path stroke];

    // state 2    
    path = [NSBezierPath bezierPath];
    [path moveToPoint: (NSPoint) {5,18}];
    [path lineToPoint: (NSPoint) {85,18}];    
    [path setLineWidth:thickness];
    [path setLineDash:dashArray2 count:3 phase: 0.0];
    [path stroke];

    // state 3    
    path = [NSBezierPath bezierPath];
    [path moveToPoint: (NSPoint) {95,18}];
    [path lineToPoint: (NSPoint) {175,18}];    
    [path setLineWidth:thickness];
    [path setLineDash:dashArray3 count:3 phase: 0.0];
    [path stroke];
}


- (void)mouseDown:(NSEvent *)theEvent // handels user input (main mouse button released)
{
    int newState=0;
    NSPoint mouseLocation = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    
    if (NSMouseInRect(mouseLocation, dash0Rect, NO))
        newState=0;
        
    if (NSMouseInRect(mouseLocation, dash1Rect, NO))
        newState=1;
        
    if (NSMouseInRect(mouseLocation, dash2Rect, NO))
        newState=2;
        
    if (NSMouseInRect(mouseLocation, dash3Rect, NO))
        newState=3;
    [self setState:newState];
}


// i want to respond to events
- (BOOL)acceptsFirstResponder {
    return YES;
}

// wenn man auf den view klickt bewegt sich nicht das fenster mit
- (BOOL)mouseDownCanMoveWindow {
    return NO;
}

- (int)state {
    return state;
}

- (void)setState: (int) flag {
    if (state != flag)
    {
        state = flag;
        [self setNeedsDisplay:YES];
    }
}

- (void) setThickness: (float) newThickness
{
    if (thickness != newThickness)
    {
        thickness = newThickness;
        [self setNeedsDisplay:YES];
    }
}

- (void) setColor: (NSColor *) newColor
{
    id oldColor = color;
    color = [newColor retain];
    [oldColor release];
    [self setNeedsDisplay:YES];
}



- (void)setWindowController:(OCEWindowController *)aController
{
    controller = aController;
}


- (void)dealloc 
{
    [color release];
    [super dealloc];
}
@end
