//
//  OCEWindowController.h
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import <AppKit/AppKit.h>
#import <WebKit/WebKit.h>
@class OCEPanelView;
@class OCECanvas;
@class OCEPanelObject;
@class OCEDocument;
@class OCETrailObject;
@class OCETrailArrowPreview;
@class OCETrailDashView;
@class OCENavFrameObject;
@class OCENavTrailObject;
@class OCEPanelView;
@class OCEImageView;

@interface OCEWindowController : NSWindowController {

    IBOutlet NSTabView *infoTabView;
    IBOutlet NSTabView *mainTabView;
	IBOutlet NSTabView *canvasTabView;
	IBOutlet WebView *webView;
	NSString *exportPath;

    // INFOCANVAS
    IBOutlet NSColorWell *infoCanvasColorWell;
    IBOutlet NSSlider *infoCanvasGridSizeSlider;
    IBOutlet NSTextField *infoCanvasGridSizeTextField;
    IBOutlet NSButton *infoCanvasShowGrid;
    IBOutlet NSButton *infoCanvasSnapToGrid;
    
    //INFOPANEL
    IBOutlet NSTextField *infoPanelWidthTextField;
    IBOutlet NSTextField *infoPanelHeightTextField;
    IBOutlet NSTextField *infoPanelXPositionTextField;
    IBOutlet NSTextField *infoPanelYPositionTextField;
	IBOutlet NSTextField *infoPanelRotationTextField;
    IBOutlet NSButton *infoPanelLockedButton;
    IBOutlet NSSlider *infoPanelTransparencySlider;

	//INFOTRAIL
    IBOutlet NSButton *infoTrailLockedButton;
    IBOutlet NSColorWell *infoTrailColorWell;
    IBOutlet NSTextField *infoTrailThicknessTextField;
    IBOutlet NSSlider *infoTrailThicknessSlider;
    IBOutlet NSMatrix *infoTrailArrowMatrix;
	IBOutlet OCETrailArrowPreview *arrowPreview1;
    IBOutlet OCETrailArrowPreview *arrowPreview2;
    IBOutlet OCETrailArrowPreview *arrowPreview3;
    IBOutlet OCETrailArrowPreview *arrowPreview4;
    IBOutlet OCETrailArrowPreview *arrowPreview5;
	
	// INFOMULTIBLESELECTION
	IBOutlet NSTextField *infoMultibleSelectionTextField;

	
    //INFONAVIGATION
	IBOutlet NSSlider *infoNavigationGridSizeSlider;
    IBOutlet NSTextField *infoNavigationGridSizeTextField;
    IBOutlet NSButton *infoNavigationShowGrid;
    IBOutlet NSButton *infoNavigationSnapToGrid;	
    IBOutlet NSMatrix *infoNavigationToolsMatrix;
    IBOutlet NSButton *infoNavigationToolsNavigationButton;
	IBOutlet NSButton *infoNavigationToolsActiveComponentHintButton;

	
    //INFOFRAME
	IBOutlet NSTextField *infoFrameWidthTextField;
	IBOutlet NSTextField *infoFrameHeightTextField;
    IBOutlet NSTextField *infoFrameXPositionTextField;
    IBOutlet NSTextField *infoFrameYPositionTextField;
	IBOutlet NSTextField *infoFrameRotationTextField;
	IBOutlet NSTextField *infoFrameURLTextField;
    IBOutlet NSButton *infoFrameLockedButton;
    IBOutlet NSButton *infoFrameStartFrameButton;
	IBOutlet NSButton *infoFrameWaitButton;
	IBOutlet NSButton *infoFrameForceAspectRatio;

    
	//INFONAVTRAIL
	IBOutlet NSButton *infoNavTrailLockedButton;
    //IBOutlet NSSlider *infoNavTrailTransitionSpeedSlider;
    IBOutlet NSPopUpButton *infoNavTrailTransitionTypePopUpButton;
	IBOutlet NSTextField *infoNavTrailTransitionSpeedTextField;
	IBOutlet NSColorWell *infoNavTrailTransitionColorWell;
	IBOutlet NSStepper *infoNavTrailTransitionSpeedStepper;
	IBOutlet NSButton *infoNavTrailEasingButton;

	
	// INFOPREVIEW
	IBOutlet NSTextField *infoPreviewWidthTextField;
    IBOutlet NSTextField *infoPreviewHeightTextField;


    // VIEWS
    IBOutlet OCEPanelView *panelView;
    IBOutlet OCECanvas *canvas;
	
	// EXPORT VIEW OUTLETS
	IBOutlet NSView			*exportView;
	IBOutlet NSTextField	*exportViewTitleTextField;
	IBOutlet NSTextField	*exportViewWidthTextField;
	IBOutlet NSTextField	*exportViewHeightTextField;
	IBOutlet NSColorWell	*exportViewBackgroundColorWell;
	IBOutlet NSColorWell	*exportViewTextColorWell;
	IBOutlet NSButton		*exportViewNavigationBarButton;

	// EXPORT PROGRESS SHEET OUTLETS
	IBOutlet NSPanel *exportProgressSheet;
	IBOutlet NSProgressIndicator *exportProgressIndicator;
	
    // MODE SELECT
    IBOutlet NSPopUpButton *displayModePopUpButton;
    
    // ZOOM
    IBOutlet NSSlider *zoomSlider;
    
    NSNotificationCenter *nc;
	BOOL exportInProgress;
	
	// InfoViews
	IBOutlet NSView *infoCanvasView;
	IBOutlet NSView *infoPanelView;
	IBOutlet NSView *infoTrailView;
	IBOutlet NSView *infoMultibleSelectionView;
	IBOutlet NSView *infoNavigationView;
	IBOutlet NSView *infoFrameView;
	IBOutlet NSView *infoNavigationTrailView;
	IBOutlet NSView *infoPreviewView;
}
// Notification
- (void) updateView;

- (BOOL) setInfoViews;

// TabViews
- (void) selectMainTabViewItemAtIndex: (int) index;
- (void) updateMainTabView;
- (void) selectInfoTabViewItemAtIndex: (int) index;

// INFOCANVAS
- (IBAction) infoCanvasColorWellAction:(id)sender;

// Info Grid
- (IBAction) infoGridSize:(id)sender;
- (IBAction) infoShowGrid:(id)sender;
- (IBAction) infoSnapToGrid:(id)sender;
- (void) setGridSize: (int) newSize;
- (void) setShowGrid: (int) flag;
- (void) setSnapToGrid: (int) flag;
- (void) setCanvasColor: (NSColor *) newColor;

// Info Object
- (IBAction) infoObjectLockedButton:(id)sender;

// Info Rect 
- (IBAction) infoRectWidthTextField:(id)sender;
- (IBAction) infoRectHeightTextField:(id)sender;
- (IBAction) infoRectXPositionTextField:(id)sender;
- (IBAction) infoRectYPositionTextField:(id)sender;
- (IBAction) infoRectRotationTextField:(id)sender;

// Info Panel
- (IBAction) infoPanelResetSize:(id)sender;
- (IBAction) infoPanelTransparencySlider:(id)sender;
- (IBAction) infoPanelRotationSlider:(id)sender;
- (void) setInfoPanel: (OCEPanelObject *) object;

// Info Trail
- (IBAction) infoTrailColorWell:(id)sender;
- (IBAction) infoTrailThicknessTextField:(id)sender;
- (IBAction) infoTrailThicknessSlider:(id)sender;
- (IBAction) infoTrailArrowMatrix:(id)sender;
- (void) setInfoTrail: (OCETrailObject *) object;

- (void) setArrow: (int) arrow;

// Info Navigation
- (IBAction) infoNavigationToolsMatrix:(id)sender;
- (IBAction) infoNavigationToolsActiveComponentHintButton:(id)sender;
- (void) setNavigationTools: (int) navigationTools;
- (void) setActiveComponentHint: (bool) flag;

// Info Frame
- (IBAction) infoFrameStartFrameButton:(id)sender;
- (IBAction) infoFrameWaitButton:(id)sender;
- (void) setInfoNavFrame: (OCENavFrameObject *) object;
- (IBAction) infoFrameCreateActiveComponentButton:(id)sender;
- (IBAction) infoFrameURLTextField:(id)sender;
- (IBAction) infoFrameForceAspectRatio:(id)sender;


// Info Nav Trail
- (IBAction) setInfoNavTrailTransitionSpeed:(id)sender;
- (IBAction) infoNavTrailTransitionTypePopUpButton:(id)sender;
- (void) setInfoNavTrail: (OCENavTrailObject *) object;
- (IBAction) infoNavTrailEasing:(id)sender;
- (IBAction) infoNavTrailTransitionColorWell:(id)sender;

// Info Preview
- (IBAction) infoPreviewReload:(id)sender;

// MODE SELECT
- (IBAction) displayModePopUpButtonAction:(id)sender;
- (void) setMode: (unsigned int) flag;

// ZOOM
- (IBAction) zoomSliderAction:(id)sender;
- (void) enableZoomSlider;
- (void) disableZoomSlider;

// Arrange Objects
- (IBAction) moveObjectUp:(id)sender;
- (IBAction) moveObjectTop:(id)sender;
- (IBAction) moveObjectDown:(id)sender;
- (IBAction) moveObjectBottom:(id)sender;
- (IBAction) gridObjects:(id)sender;

// VIEWS
- (OCECanvas *) canvas;
- (OCEPanelView *) panelView;

// Other Actions
- (IBAction) export: (id) sender;
- (IBAction) exportFlash: (id) sender;
- (void)exportPanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void)exportFlashPanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (BOOL) exportInProgress;
- (BOOL) exportToDirectory: (NSString *) path preview: (BOOL) preview;
- (BOOL) exportFlashToDirectory: (NSString *) path preview: (BOOL) preview;
- (NSString *) createHTMLSource: (NSString *) filename preview: (BOOL) preview flash: (BOOL) flash;
- (IBAction)generateNavigation:(id)sender;

// Export Progress Sheet Actions
- (void) showExportProgressSheetStartValue: (int) startValue endValue: (int) endValue;
- (void) closeExportProgressSheet;
- (void) exportProgressSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void) setExportProgress;

@end