//
//  OCEWindowController.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEWindowController.h"
#import "OCEPanelView.h"
#import "OCECanvas.h"
#import "OCEObject.h"
#import "OCEPanelObject.h"
#import "OCEDocument.h"
#import "OCETrailObject.h"
#import "OCETrailArrowPreview.h"
#import "OCETrailDashView.h"
#import "OCENavTrailObject.h"
#import "OCENavFrameObject.h"
#import "OCEImageView.h"

#import "ICFlashConverter.h"

@implementation OCEWindowController

- (void)windowDidLoad {
    [super windowDidLoad];
	//init info tab view
	[self setInfoViews];
	
	exportInProgress = NO;
	
    [panelView setWindowController:self];
    [canvas setWindowController:self];
    
    [arrowPreview1 setWindowController:self];
    [arrowPreview1 setType:0];
    [arrowPreview2 setWindowController:self];
    [arrowPreview2 setType:1];
    [arrowPreview3 setWindowController:self];
    [arrowPreview3 setType:3];
    [arrowPreview4 setWindowController:self];
    [arrowPreview4 setType:4];
    [arrowPreview5 setWindowController:self];
    [arrowPreview5 setType:6];
    [arrowPreview4 setOtherView:arrowPreview5];
    [arrowPreview5 setOtherView:arrowPreview4];
    //[self setArrowImageInPreviews];

    //[trailDashView setWindowController:self];
    [[self window] setAcceptsMouseMovedEvents:YES]; // we need this for previewPath in OCECanvas
    
    // Notifikation Stuff
    [nc addObserver: self
           selector: @selector(updateView)
               name: @"NSUndoManagerDidRedoChangeNotification"
             object: nil];
             
    [nc addObserver: self
           selector: @selector(updateView)
               name: @"NSUndoManagerDidUndoChangeNotification"
             object: nil];  
	
	[NSBundle loadNibNamed:@"Export" owner:self];
	
	[panelView setPanelCountDisplay];	
	[panelView setToolTips];
	exportPath = @" ";
}


// ----------------------------------------------------------------------
// Notification
// ----------------------------------------------------------------------
- (void) updateView
{
    [panelView setNeedsDisplay:YES];
    [canvas setNeedsDisplay:YES];
    [arrowPreview1 setNeedsDisplay:YES];
    [arrowPreview2 setNeedsDisplay:YES];
    [arrowPreview3 setNeedsDisplay:YES];
    [arrowPreview4 setNeedsDisplay:YES];
    [arrowPreview5 setNeedsDisplay:YES];
    //[trailDashView setNeedsDisplay:YES];
    
    [self updateMainTabView]; // just to make sure that the korrekt values are displayed
}


// ----------------------------------------------------------------------
// Methoden fr das Main Menu
// ----------------------------------------------------------------------
- (BOOL)validateMenuItem:(NSMenuItem*)anItem 
{
	//NSLog([anItem title]);
	
	if ([NSStringFromSelector([anItem action]) isEqualToString:@"displayModePopUpButtonAction:"]) 
		return YES;
	
	int count = [[canvas selectedObjects] count];	
    if ([[anItem title] isEqualToString:@"Move Up"] && (count != 1)) 
        return NO;
    if ([[anItem title] isEqualToString:@"Move Down"] && (count != 1)) 
        return NO;
	if ([[anItem title] isEqualToString:@"Publish"]) 
        return YES;
	if ([[anItem title] isEqualToString:@"Generate Navigation"] && ([[[self document] canvasObjects] count] > 0) && ([[[self document] navigationObjects] count] == 0)) 
        return YES;
	if (count > 0)
		return YES;
	
    return NO;
}

- (BOOL) setInfoViews;
{
	[[infoTabView tabViewItemAtIndex: 0] setView: infoCanvasView];
	[[infoTabView tabViewItemAtIndex: 1] setView: infoPanelView];
	[[infoTabView tabViewItemAtIndex: 2] setView: infoTrailView];
	[[infoTabView tabViewItemAtIndex: 3] setView: infoMultibleSelectionView];
	[[infoTabView tabViewItemAtIndex: 4] setView: infoNavigationView];
	[[infoTabView tabViewItemAtIndex: 5] setView: infoFrameView];
	[[infoTabView tabViewItemAtIndex: 6] setView: infoNavigationTrailView];
	[[infoTabView tabViewItemAtIndex: 7] setView: infoPreviewView];
	
	return YES;
}

// ----------------------------------------------------------------------
// TabViews
// ----------------------------------------------------------------------
- (void) selectMainTabViewItemAtIndex: (int) index
{
    [mainTabView selectTabViewItemAtIndex:index];
}

- (void) updateMainTabView
{
    int mode = [canvas displayMode];
    if (mode == 0)
    {
        NSMutableArray *selectedObjects = [canvas selectedObjects];
        int count = [selectedObjects count];
        if (count == 1)
        {
            OCEObject *object = [selectedObjects lastObject];
            if ([object isMemberOfClass:[OCEPanelObject class]])
                [self setInfoPanel: (OCEPanelObject *) object]; 
            else if ([object isMemberOfClass:[OCETrailObject class]])
                [self setInfoTrail: (OCETrailObject *) object]; 
            
        } // if (count == 1)
        else if (count > 1)
        {
            [infoMultibleSelectionTextField setIntValue:count];
            [infoTabView selectTabViewItemAtIndex:3]; // multiple selection
            [mainTabView selectTabViewItemAtIndex:0]; // info
        } 
        else // count == 0
        { 
            [infoTabView selectTabViewItemAtIndex:0]; // 
            [mainTabView selectTabViewItemAtIndex:0]; // info
        }
    } // if (mode == 0)
    else if (mode == 1)
    {
		NSMutableArray *selectedObjects = [canvas selectedObjects];
        int count = [selectedObjects count];
        if (count == 1)
        {
            OCEObject *object = [selectedObjects lastObject];
            if ([object isMemberOfClass:[OCENavFrameObject class]])
                [self setInfoNavFrame: (OCENavFrameObject *) object]; 
			else if ([object isMemberOfClass:[OCENavTrailObject class]])
                [self setInfoNavTrail: (OCENavTrailObject *) object]; 
            
        } // if (count == 1)
        else if (count > 1)
        {
            [infoMultibleSelectionTextField setIntValue:count];
            [infoTabView selectTabViewItemAtIndex:3]; // multiple selection
            [mainTabView selectTabViewItemAtIndex:0]; // info
        } 
        else // count == 0
        { 
            [infoTabView selectTabViewItemAtIndex:4]; // nav objects
            [mainTabView selectTabViewItemAtIndex:0]; // info
        }
    }
	else
	{
		[infoTabView selectTabViewItemAtIndex:7]; // nav objects
		[mainTabView selectTabViewItemAtIndex:0]; // info
	}
}

- (void) selectInfoTabViewItemAtIndex: (int) index
{
    [infoTabView selectTabViewItemAtIndex:index];
    [self selectMainTabViewItemAtIndex: 0];
}

// ----------------------------------------------------------------------
// Info Canvas
// ----------------------------------------------------------------------
- (IBAction)infoCanvasColorWellAction:(id)sender
{
    [[self document] setCanvasColor:[(NSColorWell *)sender color]];
}

// ----------------------------------------------------------------------
// Info Grid
// ----------------------------------------------------------------------
- (IBAction)infoGridSize:(id)sender
{
	int gridSize = [sender intValue];
	[[self document] setGridSize: gridSize];
	[infoCanvasGridSizeSlider setIntValue: gridSize];
	[infoCanvasGridSizeTextField setIntValue: gridSize];
	[infoNavigationGridSizeSlider setIntValue: gridSize];
	[infoNavigationGridSizeTextField setIntValue: gridSize];
}

- (IBAction)infoShowGrid:(id)sender
{
	[[self document] setShowGrid:[sender state]];
}

- (IBAction)infoSnapToGrid:(id)sender
{
    [[self document] setSnapToGrid:[sender state]];
}

- (void) setGridSize: (int) newSize 
{
    [infoCanvasGridSizeTextField setIntValue: newSize];
	[infoCanvasGridSizeSlider setIntValue: newSize];
	[infoNavigationGridSizeTextField setIntValue: newSize];
	[infoNavigationGridSizeSlider setIntValue: newSize];
}

- (void) setShowGrid: (int) flag 
{
    [infoCanvasShowGrid setState:flag];
	[infoNavigationShowGrid setState:flag];
}

- (void) setSnapToGrid: (int) flag 
{
    [infoCanvasSnapToGrid setState:flag];
    [infoNavigationSnapToGrid setState:flag];
}

- (void) setCanvasColor: (NSColor *) newColor 
{
    [infoCanvasColorWell setColor:newColor];
}

// ----------------------------------------------------------------------
// Info Object
// ----------------------------------------------------------------------
- (IBAction) infoObjectLockedButton:(id)sender
{
    [[[canvas selectedObjects] objectAtIndex:0] setLocked:[sender state]];
    [canvas setNeedsDisplay:YES];
}

// ----------------------------------------------------------------------
// Info Rect
// ----------------------------------------------------------------------
- (IBAction) infoRectWidthTextField:(id)sender
{
	if ([canvas objectsAreSelected]) 
	{
		[(OCERectObject *) [[canvas selectedObjects] objectAtIndex:0] setWidth: [sender intValue]];
		[canvas setNeedsDisplay:YES];
	}
}

- (IBAction) infoRectHeightTextField:(id)sender
{
	if ([canvas objectsAreSelected]) 
	{
		[(OCERectObject *) [[canvas selectedObjects] objectAtIndex:0] setHeight: [sender intValue]];
		[canvas setNeedsDisplay:YES];
	}
}

- (IBAction) infoRectXPositionTextField:(id)sender
{
	if ([canvas objectsAreSelected]) 
	{
		[(OCERectObject *) [[canvas selectedObjects] objectAtIndex:0] setOriginX: [sender intValue]];
		[canvas setNeedsDisplay:YES];
	}
}

- (IBAction) infoRectYPositionTextField:(id)sender
{
	if ([canvas objectsAreSelected]) 
	{
		[(OCERectObject *) [[canvas selectedObjects] objectAtIndex:0] setOriginY: [sender intValue]];
		[canvas setNeedsDisplay:YES];
	}
}

- (IBAction) infoRectRotationTextField:(id)sender; 
{
	if ([canvas objectsAreSelected]) 
	{
		[(OCERectObject *) [[canvas selectedObjects] objectAtIndex:0] setRotation: [sender floatValue]];
		[canvas setNeedsDisplay:YES];
	}
}


// ----------------------------------------------------------------------
// Info Panel
// ----------------------------------------------------------------------
- (IBAction) infoPanelResetSize:(id)sender
{
	[(OCEPanelObject *) [[canvas selectedObjects] objectAtIndex:0] resetSize];
    [canvas setNeedsDisplay:YES];
	[self updateMainTabView];
}

- (IBAction) infoPanelTransparencySlider:(id)sender
{
    [(OCEPanelObject *) [[canvas selectedObjects] objectAtIndex:0] setTransparency:[sender floatValue]];
    [canvas setNeedsDisplay:YES];
}

- (IBAction) infoPanelRotationSlider:(id)sender
{
    [(OCEPanelObject *) [[canvas selectedObjects] objectAtIndex:0] setRotation:[sender floatValue]];
    [canvas setNeedsDisplay:YES];
}

- (void) setInfoPanel: (OCEPanelObject *) object
{
    [infoPanelWidthTextField setIntValue: [object width]];
    [infoPanelHeightTextField setIntValue: [object height]];
    [infoPanelXPositionTextField setIntValue:[object originX]];
    [infoPanelYPositionTextField setIntValue:[object originY]];
    [infoPanelRotationTextField setIntValue:[object rotation]];
    [infoPanelLockedButton setState:[object locked]];
    [infoPanelTransparencySlider setFloatValue:[object transparency]];
    [self selectInfoTabViewItemAtIndex:1];
}

// ----------------------------------------------------------------------
// Trail
// ----------------------------------------------------------------------
- (IBAction) infoTrailColorWell:(id)sender
{
    NSColor *color = [(NSColorWell *)sender color];
    [(OCETrailObject *) [[canvas selectedObjects] objectAtIndex:0] setColor: color];
    [arrowPreview1 setColor: color];
    [arrowPreview2 setColor: color];
    [arrowPreview3 setColor: color];
    [arrowPreview4 setColor: color];
    [arrowPreview5 setColor: color];
    [canvas setNeedsDisplay:YES];
}

- (IBAction) infoTrailThicknessTextField:(id)sender;
{
	if ([canvas objectsAreSelected]) 
	{
		float thickness = [sender floatValue];
		[(OCETrailObject *) [[canvas selectedObjects] objectAtIndex:0] setThickness: thickness];
		[infoTrailThicknessSlider setFloatValue:[sender floatValue]];
		[arrowPreview1 setThickness: thickness];
		[arrowPreview2 setThickness: thickness];
		[arrowPreview3 setThickness: thickness];
		[arrowPreview4 setThickness: thickness];
		[arrowPreview5 setThickness: thickness];
		[canvas setNeedsDisplay:YES];
	}
}

- (IBAction) infoTrailThicknessSlider:(id)sender;
{
    float thickness = [sender floatValue];
    [(OCETrailObject *) [[canvas selectedObjects] objectAtIndex:0] setThickness:thickness];
    [infoTrailThicknessTextField setFloatValue:[sender floatValue]];
    [arrowPreview1 setThickness: thickness];
    [arrowPreview2 setThickness: thickness];
    [arrowPreview3 setThickness: thickness];
    [arrowPreview4 setThickness: thickness];
    [arrowPreview5 setThickness: thickness];

    [canvas setNeedsDisplay:YES];
}

- (IBAction) infoTrailArrowMatrix:(id)sender
{
    int cell = [[sender selectedCell] tag];
    int arrow = cell;
    
    switch (cell)
    {
        case 0:
        {
            arrow = 0;
            break;
        }
        case 1:
        {
            arrow = [arrowPreview2 type];
            break;
        }
        case 2:
        {
            arrow = 3;
            break;
        }
        case 3:
        {
            arrow = [arrowPreview4 type];
            break;
        }
        case 4:
        {
            arrow = 6;
            break;
        }
    }
    [(OCETrailObject *) [[canvas selectedObjects] objectAtIndex:0] setArrow: arrow];
    [canvas setNeedsDisplay:YES];
}

- (void) setInfoTrail: (OCETrailObject *) object
{
    [infoTrailLockedButton setState:[object locked]];
    [infoTrailColorWell setColor:[object color]];
    [infoTrailThicknessTextField setFloatValue:[object thickness]];
    [infoTrailThicknessSlider setFloatValue:[object thickness]];
    
    int tag=0;
    switch ([object arrow])
    {
        case 0:
        {
            tag = 0;
            break;
        }
        case 1:
        case 2:
        {
            tag = 1;
            [arrowPreview2 setType:[object arrow]];
            [arrowPreview4 setType:4];
            break;
        }
        case 3:
        {
            tag = 2;
            break;
        }
        case 4:
        case 5:
        {
            tag = 3;
            [arrowPreview4 setType:[object arrow]];
            [arrowPreview2 setType:1];
            break;
        }
        case 6:
        {
            tag = 4;
            break;
        }
    }
    [infoTrailArrowMatrix selectCellWithTag:tag];
    
    [arrowPreview1 setColor:[object color]];
    [arrowPreview1 setThickness:[object thickness]];
    [arrowPreview2 setColor:[object color]];
    [arrowPreview2 setThickness:[object thickness]];
    [arrowPreview3 setColor:[object color]];
    [arrowPreview3 setThickness:[object thickness]];
    [arrowPreview4 setColor:[object color]];
    [arrowPreview4 setThickness:[object thickness]];    
    [arrowPreview5 setColor:[object color]];
    [arrowPreview5 setThickness:[object thickness]]; 

    [self selectInfoTabViewItemAtIndex:2];
}

- (void) setArrow: (int) arrow
{
    [(OCETrailObject *) [[canvas selectedObjects] objectAtIndex:0] setArrow: arrow];
    int tag=0;
    switch (arrow)
    {
        case 0:
        {
            tag = 0;
            break;
        }
        case 1:
        case 2:
        {
            tag = 1;
            break;
        }
        case 3:
        {
            tag = 2;
            break;
        }
        case 4:
        case 5:
        {
            tag = 3;
            break;
        }
        case 6:
        {
            tag = 4;
            break;
        }
    }
    [infoTrailArrowMatrix selectCellWithTag:tag];
    [canvas setNeedsDisplay:YES];
}

// ----------------------------------------------------------------------
// Info Navigation
// ----------------------------------------------------------------------
- (IBAction) infoNavigationToolsMatrix:(id)sender;
{
	int flag = [sender selectedRow];
	[[self document] setNavigationTool: flag];
	
	if (flag)
	{
		[infoNavigationToolsNavigationButton setEnabled:YES];
		[infoNavigationToolsActiveComponentHintButton setEnabled:YES];
	}
	else
	{
		[infoNavigationToolsNavigationButton setEnabled:NO];
		[infoNavigationToolsActiveComponentHintButton setEnabled:NO];

	}
}

- (IBAction) infoNavigationToolsActiveComponentHintButton:(id)sender
{
    if ([[sender selectedCell] state] == NSOnState)
        [[self document] setActiveComponentHint: YES];
    else
        [[self document] setActiveComponentHint: NO];
}

- (void) setNavigationTools: (int) navigationTools
{
	[infoNavigationToolsMatrix selectCellAtRow: navigationTools column:0];
	if (navigationTools) {
		[infoNavigationToolsActiveComponentHintButton setEnabled:YES];
	} else {
		[infoNavigationToolsActiveComponentHintButton setEnabled:NO];
	}
}


- (void) setActiveComponentHint: (bool) flag;
{
	[infoNavigationToolsActiveComponentHintButton setState: flag];
}


// ----------------------------------------------------------------------
// Info Frame
// ----------------------------------------------------------------------
- (IBAction) infoFrameStartFrameButton:(id)sender
{
	if ([sender state])
	{
		[[self document] resetNavigationStartFrame];
		[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setIsStartFrame: YES];
	}
	else
	{
		[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setIsStartFrame: NO];
	}
}

- (IBAction) infoFrameWaitButton:(id)sender
{
	if ([sender state])
	{
		[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setWaiting: YES];
	}
	else
	{
		[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setWaiting: NO];
	}
}

- (void) setInfoNavFrame: (OCENavFrameObject *) object
{
	[infoFrameXPositionTextField setIntValue:[object originX]];
    [infoFrameYPositionTextField setIntValue:[object originY]];
    [infoFrameWidthTextField setIntValue: [object width]];
	[infoFrameHeightTextField setIntValue: [object height]];
	[infoFrameRotationTextField setIntValue:[object rotation]];
    [infoFrameLockedButton setState:[object locked]];
    [infoFrameStartFrameButton setState:[object isStartFrame]];
    [infoFrameWaitButton setState:[object waiting]];
	[infoFrameURLTextField setStringValue: [object URL]];
	[infoFrameForceAspectRatio setState: [object forceAspectRatio]];

	[self selectInfoTabViewItemAtIndex:5];
}

- (IBAction) infoFrameCreateActiveComponentButton:(id)sender
{
	[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] createActiveComponentForDocument: 
		[self document]];
	[canvas setNeedsDisplay:YES];
}

- (IBAction) infoFrameURLTextField:(id)sender
{
	[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setURL: [sender stringValue] withTarget: @"_self"];
}

- (IBAction) infoFrameForceAspectRatio:(id)sender;
{
	if ([sender state])
	{
		[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setForceAspectRatio: YES];
	}
	else
	{
		[(OCENavFrameObject *) [[canvas selectedObjects] objectAtIndex:0] setForceAspectRatio: NO];
	}
	[canvas setNeedsDisplay:YES];
	[self updateMainTabView];
}

// ----------------------------------------------------------------------
// Info Nav Trail
// ----------------------------------------------------------------------
- (IBAction) setInfoNavTrailTransitionSpeed:(id)sender;
{
	float duration = [sender floatValue];

	[infoNavTrailTransitionSpeedTextField setStringValue: [NSString stringWithFormat: @"%.2f", duration]];
	//@"%.1f" this is a workaround to ensure that the stepper maks 0.1 steps even when duration is something like 1.22
	[infoNavTrailTransitionSpeedStepper setStringValue: [NSString stringWithFormat: @"%.1f", duration]];

	[(OCENavTrailObject *) [[canvas selectedObjects] objectAtIndex:0] setTransitionSpeed: duration * 100];
}


- (IBAction) infoNavTrailTransitionTypePopUpButton:(id)sender
{
	int type = [sender indexOfSelectedItem];

	[(OCENavTrailObject *) [[canvas selectedObjects] objectAtIndex:0] setTransition: type];
	// if this is a fad through color transition, we need to enable the colorwell
	[infoNavTrailTransitionColorWell setEnabled: (type == 2) ? YES : NO];

}

- (void) setInfoNavTrail: (OCENavTrailObject *) object
{
	int type = [object transition];
    [infoNavTrailTransitionTypePopUpButton selectItemAtIndex: type];
	
	// if this is a fad through color transition, we need to enable the colorwell
	[infoNavTrailTransitionColorWell setEnabled: (type == 2) ? YES : NO];
	[infoNavTrailTransitionColorWell setColor: [object transitionColor]];
	
	float duration = [object transitionSpeed] / 100.0;
	[infoNavTrailTransitionSpeedStepper setFloatValue: duration];
	[infoNavTrailTransitionSpeedTextField setStringValue: [NSString stringWithFormat: @"%.2f", duration]];

	[infoNavTrailLockedButton setState:[object locked]];
	[infoNavTrailEasingButton setState:[object easing]];

	[self selectInfoTabViewItemAtIndex:6];
}

- (IBAction) infoNavTrailEasing:(id)sender;
{
	if ([sender state])	{
		[(OCENavTrailObject *) [[canvas selectedObjects] objectAtIndex:0] setEasing: YES];
	} else {
		[(OCENavTrailObject *) [[canvas selectedObjects] objectAtIndex:0] setEasing: NO];
	}
}

- (IBAction) infoNavTrailTransitionColorWell:(id)sender;
{
	NSColor *color = [(NSColorWell *)sender color];
    [(OCENavTrailObject *) [[canvas selectedObjects] objectAtIndex:0] setTransitionColor: color];	
}


// ----------------------------------------------------------------------
// Info Preview
// ----------------------------------------------------------------------
- (IBAction) infoPreviewReload:(id)sender
{
	[exportViewWidthTextField setIntValue: [infoPreviewWidthTextField intValue]];
	[exportViewHeightTextField setIntValue: [infoPreviewHeightTextField intValue]];
	
	NSString *filename = [[[[self document] fileName] lastPathComponent] stringByDeletingPathExtension];
	if (filename == nil)
		filename = @"ressources"; 
	
	// write html file
	NSString *htmlPath = [exportPath stringByDeletingLastPathComponent];
	htmlPath = [htmlPath stringByAppendingPathComponent: @"content.html"];
	NSString *content = [self createHTMLSource: filename preview: YES flash: YES];
	[content writeToFile: htmlPath atomically:YES];


	[webView reload:nil];
}


// ----------------------------------------------------------------------
// MODE SELECT
// ----------------------------------------------------------------------
- (IBAction) displayModePopUpButtonAction:(id)sender
{
	int displayMode = [sender indexOfSelectedItem];
	if (displayMode != [canvas displayMode]) {
		switch (displayMode) {
			case 0: 
				[canvasTabView selectTabViewItemAtIndex:0];
				[canvas setDisplayMode:0];
				break;
			case 1:
				[canvasTabView selectTabViewItemAtIndex:0];
				[canvas setDisplayMode:1];
				break;
			case 2:
				[exportViewWidthTextField setIntValue: [infoPreviewWidthTextField intValue]];
				[exportViewHeightTextField setIntValue: [infoPreviewHeightTextField intValue]];
				exportPath = [[self document] previewPath];
				[self exportToDirectory:exportPath preview: YES];
				exportPath = [[exportPath stringByAppendingPathComponent: @"index.html"] retain];
				[[webView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:[@"file:" stringByAppendingString: exportPath]]]];
				[canvasTabView selectTabViewItemAtIndex:1];
				[canvas setDisplayMode:2];
				break;
		}
	}
}

- (void) setMode: (unsigned int) flag
{
	if (flag > 2)
		return;
	
	[displayModePopUpButton selectItemAtIndex:flag]; 
	[canvas setDisplayMode:flag];
}

// ----------------------------------------------------------------------
// Zoom
// ----------------------------------------------------------------------
- (IBAction) zoomSliderAction:(id)sender {
	float factor = [sender floatValue];
	
	if (factor == 5)
		factor = 1;
	else if (factor > 5) {
		factor -= 4;
	}
	else if (factor < 5)
		factor /= 5;
    [canvas scaleCanvas:factor];
}

- (void) enableZoomSlider
{
	[zoomSlider setEnabled: YES];
}

- (void) disableZoomSlider
{
	[zoomSlider setEnabled: NO];
}

// ----------------------------------------------------------------------
// Arrange Objects
// ----------------------------------------------------------------------
- (IBAction) moveObjectUp:(id)sender
{
	[[self document] moveObjectUp: [[canvas selectedObjects] objectAtIndex:0]];
}

- (IBAction) moveObjectTop:(id)sender
{
	[[self document] moveObjectsTop: [canvas selectedObjects]];
}

- (IBAction) moveObjectDown:(id)sender
{
	[[self document] moveObjectDown: [[canvas selectedObjects] objectAtIndex:0]];
}

- (IBAction) moveObjectBottom:(id)sender
{
	[[self document] moveObjectsBottom: [canvas selectedObjects]];
}

- (IBAction) gridObjects:(id)sender
{
	NSArray *array = [canvas selectedObjects];
	int i, c = [array count];
	OCEObject *object;
	for (i=0;i<c;i++)
	{
		object = [array objectAtIndex:i];
		[object startManipulationWithSelector:@selector(moveXBy:moveYBy:) andData: nil];
		[object gridObject: [[self document] gridSize]];
	}
	[array makeObjectsPerformSelector:@selector(stopManipulation)];
	[self updateMainTabView];
	[canvas setNeedsDisplay:YES];
}

// ----------------------------------------------------------------------
// VIEWS
// ----------------------------------------------------------------------
- (OCECanvas *) canvas
{
    return canvas;
}

- (OCEPanelView *) panelView
{
	return panelView;
}

- (void)dealloc 
{
    [nc removeObserver: self];
    [super dealloc];
}

// ----------------------------------------------------------------------
// Other Actions
// ----------------------------------------------------------------------
- (IBAction) export: (id) sender
{
	NSOpenPanel *myPanel = [NSOpenPanel openPanel];
	if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
		[myPanel setCanCreateDirectories:YES];	
	[myPanel setCanChooseDirectories:YES];
	[myPanel setAllowsMultipleSelection:NO];
	[myPanel setCanChooseFiles:NO];
	[myPanel setPrompt: @"Publish"];
	[myPanel setAccessoryView: exportView];
	
	[myPanel beginSheetForDirectory: nil
							   file: nil
					 modalForWindow: [self window]
					  modalDelegate: self
					 didEndSelector: @selector(exportPanelDidEnd:returnCode:contextInfo:)
						contextInfo: nil];
}

- (IBAction) exportFlash: (id) sender 
{
	NSOpenPanel *myPanel = [NSOpenPanel openPanel];
	if (!(floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_2))
		[myPanel setCanCreateDirectories:YES];	
	[myPanel setCanChooseDirectories:YES];
	[myPanel setAllowsMultipleSelection:NO];
	[myPanel setCanChooseFiles:NO];
	[myPanel setPrompt: @"Publish"];
	[myPanel setAccessoryView: exportView];
	
	[myPanel beginSheetForDirectory: nil
							   file: nil
					 modalForWindow: [self window]
					  modalDelegate: self
					 didEndSelector: @selector(exportFlashPanelDidEnd:returnCode:contextInfo:)
						contextInfo: nil];
	
}


- (void)exportFlashPanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
{
    [sheet orderOut:self];
	if (returnCode == NSOKButton) {
		NSString *directory = [sheet directory];
		[self exportFlashToDirectory: directory preview: NO];
    }
}

- (void)exportPanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
{
    [sheet orderOut:self];
	if (returnCode == NSOKButton) {
		NSString *directory = [sheet directory];
		[self exportToDirectory: directory preview: NO];
    }
}

- (BOOL) exportToDirectory: (NSString *) path preview: (BOOL) preview;
{	
	return [self exportFlashToDirectory: path preview: preview];
}

- (BOOL) exportFlashToDirectory: (NSString *) path preview: (BOOL) preview;
{
	// extract document name
	NSString *filename = [[[[self document] fileName] lastPathComponent] stringByDeletingPathExtension];
	if (filename == nil)
		filename = @"ressources"; 
	NSString *ressourcePath = [path stringByAppendingPathComponent: filename];
	NSFileManager *manager = [NSFileManager defaultManager];
	[manager createDirectoryAtPath: ressourcePath  attributes:nil]; // create ressource Dir
	
	// write html file
	NSString *htmlPath = [path stringByAppendingPathComponent: @"content.html"];
	NSString *content = [self createHTMLSource: filename preview: preview flash: YES];
	if (![content writeToFile: htmlPath atomically:YES])
		return NO;
	
	// copy index.html
	NSString *source = [[NSBundle mainBundle] pathForResource:@"index" ofType:@"html"];
	[manager copyPath: source toPath: [path stringByAppendingPathComponent: [source lastPathComponent]] handler: nil];
	
	// copy alternate.html
	source = [[NSBundle mainBundle] pathForResource:@"alternate" ofType:@"html"];
	[manager copyPath: source toPath: [path stringByAppendingPathComponent: [source lastPathComponent]] handler: nil];

	// copy alternate.gif
	source = [[NSBundle mainBundle] pathForResource:@"alternate" ofType:@"gif"];
	[manager copyPath: source toPath: [path stringByAppendingPathComponent: [source lastPathComponent]] handler: nil];
	
	// copy ICViewer.swf
	source = [[NSBundle mainBundle] pathForResource:@"ICViewer" ofType:@"swf"];
	[manager copyPath: source toPath: [path stringByAppendingPathComponent: [source lastPathComponent]] handler: nil];

	// copy flash_detection.swf
	source = [[NSBundle mainBundle] pathForResource:@"flash_detection" ofType:@"swf"];
	[manager copyPath: source toPath: [path stringByAppendingPathComponent: [source lastPathComponent]] handler: nil];

	// export document
	[[self document] exportFlashTo: ressourcePath];	
	
	return true;
}


- (NSString *) createHTMLSource: (NSString *) filename preview: (BOOL) preview flash: (BOOL) flash;
{
	int r, g, b, bgR, bgG, bgB;
	
	NSColor *color = [exportViewBackgroundColorWell color];
	if (![[color colorSpaceName] isEqualToString:@"NSCalibratedRGBColorSpace"])
		if ((color = [color colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"]) == nil)
			color = [[NSColor blackColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];
	bgR = [color redComponent] * 255;
	bgG = [color greenComponent] * 255;
	bgB = [color blueComponent] * 255;
	
	
	NSString *output = [NSString stringWithFormat:
		@"<HTML>\n<HEAD>\n<TITLE>%@</TITLE>\n</HEAD>\n\n<BODY BGCOLOR=#%x%x%x ", 
		[exportViewTitleTextField stringValue], bgR, bgG, bgB];
	
	color = [exportViewTextColorWell color];
	if (![[color colorSpaceName] isEqualToString:@"NSCalibratedRGBColorSpace"])
		if ((color = [color colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"]) == nil)
			color = [[NSColor whiteColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];		
	r = [color redComponent] * 255;
	g = [color greenComponent] * 255;
	b = [color blueComponent] * 255;

	if (preview) {
		output = [output stringByAppendingFormat: @"TEXT=#%x%x%x>\n\n<CENTER>\n",
			r,g,b,[exportViewTitleTextField stringValue]];
	} else {
		output = [output stringByAppendingFormat: @"TEXT=#%x%x%x>\n\n<CENTER>\n<br><h1>%@</h1><br>\n\n",
			r,g,b,[exportViewTitleTextField stringValue]];
	}

	BOOL useNavigationButtons = NO;
	if (preview || ([[exportViewNavigationBarButton selectedCell] state] == NSOnState)) {
		useNavigationButtons = YES;
	}
	
	if (flash) {
		NSString *bgColor = [NSString stringWithFormat: @"#%x%x%x", bgR, bgG, bgB];
		output = [output stringByAppendingFormat: @"<object classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" codebase=\"http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0\" width=\"%i\" height=\"%i\" id=\"ICViewer\" align=\"middle\">\n", 
			[exportViewWidthTextField intValue], (useNavigationButtons ? [exportViewWidthTextField intValue] + 40 : [exportViewWidthTextField intValue])];
		output = [output stringByAppendingString: @"<param name=\"allowScriptAccess\" value=\"sameDomain\" />\n"];
		output = [output stringByAppendingString: @"<param name=\"movie\" value=\"ICViewer.swf\" />\n"];
		output = [output stringByAppendingFormat: @"<param name=\"FlashVars\" value=\"comic=%@&navigationbar=%@\" />\n", filename, (useNavigationButtons ? @"true" : @"false")];
		output = [output stringByAppendingString: @"<param name=\"quality\" value=\"high\" />\n"];
		output = [output stringByAppendingFormat: @"<param name=\"bgcolor\" value=\"%@\" />\n", bgColor];
		output = [output stringByAppendingFormat: @"<embed src=\"ICViewer.swf\" FlashVars=\"comic=%@&navigationbar=%@\" quality=\"high\" bgcolor=\"%@\" width=\"%i\" height=\"%i\" name=\"ICViewer\" align=\"middle\" allowScriptAccess=\"sameDomain\" type=\"application/x-shockwave-flash\" pluginspage=\"http://www.macromedia.com/go/getflashplayer\" />\n", 
			filename, (useNavigationButtons ? @"true" : @"false"), bgColor, [exportViewWidthTextField intValue], 
			(useNavigationButtons ? [exportViewWidthTextField intValue] + 40 : [exportViewWidthTextField intValue])];
		output = [output stringByAppendingString: @"</object>\n"];
	} else {
		// java
	
		output = [output stringByAppendingString: @"<APPLET archive=\"InfiniteCanvasApplet.jar\" codebase=\".\" "];
		output = [output stringByAppendingFormat: @"code=\"InfiniteCanvasApplet.class\" width=%i height=%i>\n",
			[exportViewWidthTextField intValue],
			[exportViewWidthTextField intValue] + 40];
		output = [output stringByAppendingFormat: @"<PARAM name=\"document\" value=\"%@\">\n", filename];
		output = [output stringByAppendingFormat: @"<PARAM name=\"R\" value=\"%i\">\n", (useNavigationButtons ? 255 : bgR)];
		output = [output stringByAppendingFormat: @"<PARAM name=\"G\" value=\"%i\">\n", (useNavigationButtons ? 255 : bgG)];		
		output = [output stringByAppendingFormat: @"<PARAM name=\"B\" value=\"%i\">\n", (useNavigationButtons ? 255 : bgB)];
		output = [output stringByAppendingFormat: @"<PARAM name=\"NavigationButtons\" value=\"%@\">\n", (useNavigationButtons ? @"YES" : @"NO")];
		output = [output stringByAppendingFormat: @"<b>Your browser does not support Java, so nothing is displayed.</b>\n</APPLET>\n"];
	}
	
	return [output stringByAppendingString: @"</CENTER>\n\n</BODY>\n</HTML>"];
}

- (BOOL) exportInProgress; {
	return exportInProgress;
}


/* 
autoGenerateNavigationFrameNavigation;
*/
- (IBAction)generateNavigation:(id)sender {
	/*[[self document] autoGenerateNavigationFrameNavigation: [infoNavigationAppletWidthTextField intValue]];
	[self setNavigationTools:1];
	[self setMode:1];*/
}

// ----------------------------------------------------------------------
// Export Progress Sheet Actions
// ----------------------------------------------------------------------
- (void) showExportProgressSheetStartValue: (int) startValue endValue: (int) endValue;
{
    if (!exportProgressSheet)
        [NSBundle loadNibNamed: @"ExportProgress" owner: self];
		
	[exportProgressIndicator setMinValue: startValue];
	[exportProgressIndicator setMaxValue: endValue];
	[exportProgressIndicator setDoubleValue: startValue];
		
    [NSApp beginSheet: exportProgressSheet
            modalForWindow: [self window]
            modalDelegate: self
            didEndSelector: @selector(exportProgressSheetDidEnd:returnCode:contextInfo:)
            contextInfo: nil];
}

- (void) closeExportProgressSheet;
{
    [NSApp endSheet:exportProgressSheet];
}

- (void) exportProgressSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
{
    [sheet orderOut:self];
}

- (void) setExportProgress;
{
	[exportProgressIndicator incrementBy: 1];
}

@end
